@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen 
+# Copyright (C) 2008-2009 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,13 +15,13 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 #
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
+# The developers of the Exaile media player hereby grant permission
+# for non-GPL compatible GStreamer and Exaile plugins to be used and
+# distributed together with GStreamer and Exaile. This permission is
+# above and beyond the permissions granted by the GPL license by which
+# Exaile is covered. If you modify this code, you may extend this
+# exception to your version of the code, but you are not obligated to
+# do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
 """
@@ -34,14 +34,20 @@ A library finds tracks in a specified directory and adds them to an associated
 collection.
 """
 
+import os
+import time
+import os.path
+import shutil
+import logging
+from collections import deque
+
+import gobject
+import gio
+
 from xl.nls import gettext as _
-from xl import trackdb, track, common, xdg, event, metadata, settings
+from xl import common, xdg, event, metadata, settings, trax
 from xl.settings import SettingsManager
 
-import os, time, os.path, shutil, logging
-import gobject
-import urllib
-import traceback
 
 logger = logging.getLogger(__name__)
 
@@ -51,7 +57,7 @@ COLLECTIONS = set()
 def get_collection_by_loc(loc):
     """
         gets the collection by a location.
-        
+
         :param loc: Location of the collection
         :return: collection at location or None
         :rtype: Collection
@@ -61,7 +67,7 @@ def get_collection_by_loc(loc):
             return c
     return None
 
-class Collection(trackdb.TrackDB):
+class Collection(trax.TrackDB):
     """
         Manages a persistent track database.
 
@@ -76,7 +82,7 @@ class Collection(trackdb.TrackDB):
         >>> tracks = list(c.search('artist="TestArtist"'))
         >>> print len(tracks)
         5
-        >>> 
+        >>>
     """
     def __init__(self, name, location=None, pickle_attrs=[]):
         global COLLECTIONS
@@ -88,7 +94,7 @@ class Collection(trackdb.TrackDB):
         self._frozen = False
         self._libraries_dirty = False
         pickle_attrs += ['_serial_libraries']
-        trackdb.TrackDB.__init__(self, name, location=location,
+        trax.TrackDB.__init__(self, name, location=location,
                 pickle_attrs=pickle_attrs)
         COLLECTIONS.add(self)
 
@@ -127,8 +133,6 @@ class Collection(trackdb.TrackDB):
         if loc not in self.libraries:
             self.libraries[loc] = library
             library.set_collection(self)
-        else:
-            traceback.print_exc()
         self.serialize_libraries()
         self._dirty = True
 
@@ -148,7 +152,7 @@ class Collection(trackdb.TrackDB):
             if v == library:
                 del self.libraries[k]
                 break
- 
+
         to_rem = []
         if not "://" in library.location:
             location = u"file://" + library.location
@@ -157,8 +161,8 @@ class Collection(trackdb.TrackDB):
         for tr in self.tracks:
             if tr.startswith(location):
                 to_rem.append(self.tracks[tr]._track)
-        self.remove_tracks(to_rem)       
-       
+        self.remove_tracks(to_rem)
+
         self.serialize_libraries()
         self._dirty = True
 
@@ -172,10 +176,10 @@ class Collection(trackdb.TrackDB):
             Stops the library scan
         """
         self._scan_stopped = True
-    
+
     def get_libraries(self):
         """
-            Gets a list of all the Libraries associated with this 
+            Gets a list of all the Libraries associated with this
             Collection
 
             :rtype: list of :class:`Library`
@@ -195,18 +199,11 @@ class Collection(trackdb.TrackDB):
         self._scanning = True
         self._scan_stopped = False
 
-        self.file_count = 0
-        for library in self.libraries.values():
-            if self._scan_stopped: 
-                self._scanning = False
-                return
-            self.file_count += library._count_files()
+        self.file_count = -1 # negative means we dont know it yet
 
-        logger.info("File count: %d" % self.file_count)
+        self.__count_files()
 
-        scan_interval = self.file_count / len(self.libraries.values()) / 100
-        if not scan_interval: 
-            scan_interval = 1
+        scan_interval = 20
 
         for library in self.libraries.values():
             event.add_callback(self._progress_update, 'tracks_scanned',
@@ -215,19 +212,31 @@ class Collection(trackdb.TrackDB):
             event.remove_callback(self._progress_update, 'tracks_scanned',
                 library)
             self._running_total_count += self._running_count
-            if self._scan_stopped: 
+            if self._scan_stopped:
                 break
         else: # didnt break
             try:
                 self.save_to_location()
             except AttributeError:
-                traceback.print_exc()
+                common.log_exception(log=logger)
 
         event.log_event('scan_progress_update', self, 100)
 
         self._running_total_count = 0
         self._running_count = 0
         self._scanning = False
+        self.file_count = -1
+
+    @common.threaded
+    def __count_files(self):
+        file_count = 0
+        for library in self.libraries.values():
+            if self._scan_stopped:
+                self._scanning = False
+                return
+            file_count += library._count_files()
+        self.file_count = file_count
+        logger.debug("File count: %s"%self.file_count)
 
     def _progress_update(self, type, library, count):
         """
@@ -237,8 +246,9 @@ class Collection(trackdb.TrackDB):
         self._running_count = count
         count = count + self._running_total_count
 
-        if self.file_count == 0:
-            event.log_event('scan_progress_update', self, 100)
+        if self.file_count < 0:
+            event.log_event('scan_progress_update', self, 0)
+            return
 
         try:
             event.log_event('scan_progress_update', self,
@@ -279,141 +289,12 @@ class Collection(trackdb.TrackDB):
             closing network connections, etc.
         """
         #TODO: make close() part of trackdb
-        collections.remove(self)
+        COLLECTIONS.remove(self)
 
     def delete_tracks(self, tracks):
         for tr in tracks:
             for prefix, lib in self.libraries.iteritems():
-                if tr['__loc'].startswith('file://%s'%prefix):
-                    lib.delete(tr['__loc'])
-
-
-class ProcessEvent(object):
-    """
-        Stub ProcessEvent.  This is here so that the
-        :class:`INotifyEventProcessor` declaration doesn't fail if
-        :mod:`pyinotify` doesn't import correctly
-    """
-    pass
-
-# attempt to import pyinotify
-try:
-    import pyinotify
-    from pyinotify import EventsCodes, ProcessEvent
-except ImportError:
-    pyinotify = None
-except:
-    pyinotify = None
-    import traceback
-    traceback.print_exc()
-
-class INotifyEventProcessor(ProcessEvent):
-    """
-        Processes events from inotify
-    """
-    def __init__(self):
-        """
-            Initializes the Event Processor
-        """
-        self.libraries = []
-        try:
-            # pyinotify > 0.8
-            self.mask = pyinotify.IN_MOVED_TO|pyinotify.IN_MOVED_FROM|\
-                pyinotify.IN_CREATE|pyinotify.IN_DELETE|\
-                pyinotify.IN_CLOSE_WRITE
-        except AttributeError:
-            # pyinotify < 0.8
-            self.mask = EventsCodes.IN_MOVED_TO|EventsCodes.IN_MOVED_FROM|\
-                EventsCodes.IN_CREATE|EventsCodes.IN_DELETE|\
-                EventsCodes.IN_CLOSE_WRITE
-            
-        self.wm = pyinotify.WatchManager()
-        self.notifier = pyinotify.ThreadedNotifier(self.wm, self)
-        self.notifier.setDaemon(True)
-        self.started = False
-
-    def add_library(self, library):
-        """
-            Adds a library to be monitored by inotify.  If the ThreadedNotifier
-            hasn't been started already, it will be started
-
-            :param library:  the library to be watched
-            :type library: :class:`Library`
-        """
-        wdd = self.wm.add_watch(library.location,
-            self.mask, rec=True, auto_add=True)
-
-        self.libraries.append((library, wdd))
-        logger.info("Watching directory: %s" % library.location)
-        if not self.started:
-            self.notifier.start()
-            self.started = True
-
-    def remove_library(self, library):
-        """
-            Stops a library from being watched.  If after removing the
-            specified library there are no more libraries being watched, the
-            notifier thread is stopped
-
-            :param library: the library to remove
-            :type library: :class:`Library`
-        """
-        if not self.libraries: return
-        i = 0
-        for (l, wdd) in self.libraries:
-            if l == library:
-                self.wm.rm_watch(wdd[l.location], rec=True)        
-                break
-            i += 1
-
-
-        self.libraries.pop(i)
-        if not self.libraries:
-            self.notifier.stop()
-
-    def process_IN_DELETE(self, event):
-        """
-            Called when a file is deleted
-        """
-        pathname = os.path.join(event.path, event.name)
-        logger.info("Location deleted: %s" % pathname)
-        for (library, wdd) in self.libraries:
-            if pathname.find(library.location) > -1:
-                library._remove_locations([pathname])         
-                break
-
-    def process_IN_CLOSE_WRITE(self, event):
-        """
-            Called when a file is changed
-        """
-        pathname = os.path.join(event.path, event.name)
-        logger.info("Location modified: %s" % pathname)
-        for (library, wdd) in self.libraries:
-            if pathname.find(library.location) > -1:
-                library._scan_locations([pathname])         
-                break
-
-    def process_IN_MOVED_FROM(self, event):
-        """
-            Called when a file is created as the result of moving
-        """
-        self.process_IN_DELETE(event)
-
-    def process_IN_MOVED_TO(self, event):
-        """
-            Called when a file is removed as the result of moving
-        """
-        self.process_IN_CLOSE_WRITE(event)
-
-if pyinotify:
-    EVENT_PROCESSOR = INotifyEventProcessor()
-
-class PyInotifyNotSupportedException(Exception):
-    """
-        Thrown when set_realtime is called with True and pyinotify is not
-        installed or not supported
-    """
-    pass
+                lib.delete(tr.get_loc_for_io())
 
 class Library(object):
     """
@@ -437,7 +318,7 @@ class Library(object):
         ./tests/data
         >>> print len(list(c.search('artist="TestArtist"')))
         5
-        >>> 
+        >>>
     """
     def __init__(self, location, realtime=False, scan_interval=0):
         """
@@ -448,13 +329,6 @@ class Library(object):
         self.scan_id = 0
         self.scanning = False
         self.realtime = False
-        try:
-            self.set_realtime(realtime)
-        except PyInotifyNotSupportedException:
-            logger.warning("PyInotify not installed or not supported. " +
-                "Not watching library: %s" % location)
-        except:
-            common.log_exception()
 
         self.collection = None
         self.set_rescan_interval(scan_interval)
@@ -499,8 +373,8 @@ class Library(object):
                     tr.read_tags()
                     continue
 
-            tr = track.Track(fullpath)
-            if tr._scan_valid == True:
+            tr = trax.Track(fullpath)
+            if tr._scan_valid:
                 db.add(tr)
 
     def _remove_locations(self, locations):
@@ -512,31 +386,22 @@ class Library(object):
         for loc in locations:
             try:
                 track = self.collection.tracks[loc]
-            except KeyError:    
-                traceback.print_exc()
+            except KeyError:
+                common.log_exception(log=logger)
                 continue
 
             self.collection.remove(track)
 
-    def _walk(self, location):
-        """
-            Compatibility wrapper for Python 2.5
-        """
-        try:
-            return os.walk(self.location, followlinks=True)
-        except TypeError:
-            return os.walk(self.location)
-
     def _count_files(self):
         """
             Counts the number of files present in this directory
         """
         count = 0
-        for folder in self._walk(self.location):
+        for file in self._walk(gio.File(self.location)):
             if self.collection:
-                if self.collection._scan_stopped: 
-                    return
-            count += len(folder[2])
+                if self.collection._scan_stopped:
+                    break
+            count += 1
 
         return count
 
@@ -554,15 +419,23 @@ class Library(object):
                 to this list
             :param tr: the track to check
         """
-        # TODO: make this optional, probably in the advanced configuration
-        # editor
         # check for compilations
         if not settings.get_option('collection/file_based_compilations', True):
-            return 
+            return
+
+        def joiner(value):
+            if not value or type(value) in (str, unicode):
+                return value
+            else:
+                try:
+                    return u"\u0000".join(value)
+                except TypeError:
+                    return value
+
         try:
-            basedir = metadata.j(tr['__basedir'])
-            album = metadata.j(tr['album'])
-            artist = metadata.j(tr['artist'])
+            basedir = joiner(tr.get_tag_raw('__basedir'))
+            album = joiner(tr.get_tag_raw('album'))
+            artist = joiner(tr.get_tag_raw('artist'))
         except UnicodeDecodeError: #TODO: figure out why this happens
             logger.warning("Encoding error, skipping compilation check")
             return
@@ -574,121 +447,189 @@ class Library(object):
                 ccheck[basedir] = {}
 
             if not album in ccheck[basedir]:
-                ccheck[basedir][album] = []
+                ccheck[basedir][album] = deque()
         except TypeError:
-            traceback.print_exc()
+            common.log_exception(log=logger)
             return
 
         if ccheck[basedir][album] and not \
             artist in ccheck[basedir][album]:
             if not (basedir, album) in compilations:
                 compilations.append((basedir, album))
-            logger.info("Compilation detected: %s" % ((basedir, album),))
+                logger.info("Compilation %(album)s detected in %(dir)s" %
+                        {'album':album, 'dir':basedir})
 
         ccheck[basedir][album].append(artist)
 
+    def _walk(self, root):
+        """
+            Walk through a gio directory, returning each file in turn.
+
+            Files are enumerated in the following order: first the directory,
+            then the files in that directory. Once one directory's files have
+            all been listed, it moves on to the next directory. Order of files
+            within a dir and order of dir traversal is not specified.
+
+            :root: A gio.File representing the directory to walk through
+
+            returns a generator object that yields each gio.File in turn
+        """
+        queue = deque()
+        queue.append(root)
+
+        while len(queue) > 0:
+            dir = queue.pop()
+            yield dir
+            try:
+                for fileinfo in dir.enumerate_children("standard::type,"
+                        "standard::is-symlink,standard::name,"
+                        "standard::symlink-target,time::modified"):
+                    fil = dir.get_child(fileinfo.get_name())
+                    # FIXME: recursive symlinks could cause an infinite loop
+                    if fileinfo.get_is_symlink():
+                        target = fileinfo.get_symlink_target()
+                        if not "://" in target and not os.path.isabs(target):
+                            fil2 = dir.get_child(target)
+                        else:
+                            fil2 = gio.File(target)
+                        # already in the collection, we'll get it anyway
+                        if fil2.has_prefix(root):
+                            continue
+                    type = fileinfo.get_file_type()
+                    if type == gio.FILE_TYPE_DIRECTORY:
+                        queue.append(fil)
+                    elif type == gio.FILE_TYPE_REGULAR:
+                        yield fil
+            except gio.Error: # why doesnt gio offer more-specific errors?
+                pass
+
+    def update_track(self, gloc):
+        """
+            Rescan the track at a given location
+
+            :gloc: A gio.File representing the location
+
+            returns: the Track object, None if it could not be updated
+        """
+        uri = gloc.get_uri()
+        if not uri: # we get segfaults if this check is removed
+            return None
+        mtime = gloc.query_info("time::modified").get_modification_time()
+        tr = self.collection.get_track_by_loc(uri)
+        if tr:
+            if tr.get_tag_raw('__modified') < mtime:
+                tr.read_tags()
+                tr.set_tag_raw('__modified', mtime)
+        else:
+            tr = trax.Track(uri)
+            if tr._scan_valid == True:
+                tr.set_tag_raw('__date_added', time.time())
+                self.collection.add(tr)
+                tr.set_tag_raw('__modified', mtime)
+
+            # Track already existed. This fixes trax.get_tracks_from_uri
+            # on windows, unknown why fix isnt needed on linux.
+            elif not tr._init:
+                self.collection.add(tr)
+        return tr
+
     def rescan(self, notify_interval=None):
         """
             Rescan the associated folder and add the contained files
             to the Collection
         """
+        # TODO: use gio's cancellable support
+
         if self.collection is None:
             return True
 
-        if self.scanning: return
+        if self.scanning:
+            return
 
         logger.info("Scanning library: %s" % self.location)
         self.scanning = True
-        formats = metadata.formats.keys()
         db = self.collection
-
-        compilations = []
-        ccheck = {} # compilations dict
+        libloc = gio.File(self.location)
 
         count = 0
-        for basepath, dirnames, filenames in self._walk(self.location):
-            for filename in filenames:
-                if self.collection:
-                    if self.collection._scan_stopped: 
-                        self.scanning = False
-                        return False
-                count += 1
-                path = os.path.abspath(os.path.join(basepath, filename))
-                if os.path.islink(path):
-                    link_loc = os.readlink(path)
-                    if not link_loc.startswith(os.sep):
-                        link_loc = os.path.realpath(os.path.join(os.path.dirname(path), link_loc))
-                    if link_loc.startswith(os.path.realpath(self.location)):
-                        logger.info("Ignoring symlink %(path)s "
-                            "pointing to %(location)s" % {
-                                'path': path,
-                                'location': link_loc
-                            })
-                        continue
+        dirtracks = deque()
+        compilations = deque()
+        ccheck = {}
+        for fil in self._walk(libloc):
+            count += 1
+            type = fil.query_info("standard::type").get_file_type()
+            if type == gio.FILE_TYPE_DIRECTORY:
+                if dirtracks:
+                    for tr in dirtracks:
+                        self._check_compilation(ccheck, compilations, tr)
+                    for (basedir, album) in compilations:
+                        base = basedir.replace('"', '\\"')
+                        alb = album.replace('"', '\\"')
+                        items = [ tr for tr in dirtracks if \
+                                tr.get_tag_raw('__basedir') == base and \
+                                # FIXME: this is ugly
+                                alb in "".join(
+                                    tr.get_tag_raw('album') or []).lower()
+                                ]
+                        for item in items:
+                            item.set_tag_raw('__compilation', (basedir, album))
+                dirtracks = deque()
+                compilations = deque()
+                ccheck = {}
+            elif type == gio.FILE_TYPE_REGULAR:
+                tr = self.update_track(fil)
+                if not tr:
+                    continue
 
-                fullpath = "file://" + path
+                if dirtracks is not None:
+                    dirtracks.append(tr)
+                    # do this so that if we have, say, a 4000-song folder
+                    # we dont get bogged down trying to keep track of them
+                    # for compilation detection. Most albums have far fewer
+                    # than 110 tracks anyway, so it is unlikely that this
+                    # restriction will affect the heuristic's accuracy.
+                    # 110 was chosen to accomodate "top 100"-style
+                    # compilations.
+                    if len(dirtracks) > 110:
+                        logger.info("Too many files, skipping "
+                                "compilation detection heuristic.")
+                        dirtracks = None
 
-                try:
-                    trmtime = db.get_track_attr(fullpath, '__modified')
-                except TypeError:
-                    pass
-                except:
-                    traceback.print_exc()
-                else:
-                    mtime = os.path.getmtime(path)
-                    if mtime == trmtime:
-                        continue
+            if self.collection and self.collection._scan_stopped:
+                self.scanning = False
+                return
 
-                tr = db.get_track_by_loc(fullpath)
-                if tr:
-                    tr.read_tags()
-                else:
-                    tr = track.Track(fullpath)
-                    if tr._scan_valid == True:
-                        tr['__date_added'] = time.time()
-                        db.add(tr)
-
-                self._check_compilation(ccheck, compilations, tr)
-
-                # notify scanned tracks
-                if notify_interval is not None:
-                    if count % notify_interval == 0:
-                        event.log_event('tracks_scanned', self, count)
-
-        for (basedir, album) in compilations:
-            base = basedir.replace('"', '\\"')
-            alb = album.replace('"', '\\"')
-            items = db.search('__basedir="%s" album="%s"' % (base, alb))
-            for item in items:
-                item['__compilation'] = (basedir, album)
+            # progress update
+            if notify_interval is not None and count % notify_interval == 0:
+                event.log_event('tracks_scanned', self, count)
 
+        # final progress update
         if notify_interval is not None:
             event.log_event('tracks_scanned', self, count)
 
-        removals = []
-        location = self.location
-        if "://" not in location:
-            location = u"file://" + location
 
-        for k, tr in db.tracks.iteritems():
+
+        removals = deque()
+        for k, tr in self.collection.tracks.iteritems():
             tr = tr._track
             loc = tr.get_loc_for_io()
-            if not loc: continue
+            if not loc:
+                continue
+            gloc = gio.File(loc)
             try:
-                if not loc.startswith(location):
+                if not gloc.has_prefix(libloc):
                     continue
             except UnicodeDecodeError:
+                common.log_exception(log=logger)
                 continue
 
-            if not os.path.exists(loc.replace('file://', '')):
+            if not gloc.query_exists():
                 removals.append(tr)
-       
-        for tr in removals:
-            logger.debug(u"Removing " + unicode(tr))
-            db.remove(tr)
 
+        for tr in removals:
+            logger.debug(u"Removing %s"%unicode(tr))
+            self.collection.remove(tr)
         self.scanning = False
-        return True
 
     def is_realtime(self):
         """
@@ -703,15 +644,7 @@ class Library(object):
             :raises PyInotifyNotSupportedException: The collection is not set
                 to realtime or :mod:`pyinotify` is not available
         """
-        if realtime and not pyinotify:
-            raise PyInotifyNotSupportedException()           
-
-        self.realtime = realtime
-        if realtime and pyinotify:
-            EVENT_PROCESSOR.add_library(self)
-        
-        if not realtime and pyinotify:
-            EVENT_PROCESSOR.remove_library(self)
+        pass # TODO: reimplement with gio.FileMonitor
 
     def get_rescan_interval(self):
         """
@@ -735,28 +668,26 @@ class Library(object):
             if self.scan_id:
                 gobject.source_remove(self.scan_id)
 
-            self.scan_id = gobject.timeout_add(interval * 1000, 
-                self.rescan)
+            self.scan_id = gobject.timeout_add_seconds(interval, self.rescan)
 
         self.scan_interval = interval
 
     def add(self, loc, move=False):
         """
-            Copies (or moves) a file into the library and adds it to the 
+            Copies (or moves) a file into the library and adds it to the
             collection
         """
-        if not loc.startswith("file://"):
-            return
-        loc = loc[7:]
-        print self.location
+        oldgloc = gio.File(loc)
+
+        newgloc = gio.File(self.location).resolve_relative_path(
+                oldgloc.get_basename())
 
-        newloc = os.path.join(self.location, os.path.basename(loc))
         if move:
-            shutil.move(loc, newloc)
+            oldgloc.move(newgloc)
         else:
-            shutil.copy(loc, newloc)
-        tr = track.Track(newloc)
-        if tr._scan_valid == True:
+            oldgloc.copy(newgloc)
+        tr = trax.Track(newgloc.get_uri())
+        if tr._scan_valid:
             self.collection.add(tr)
 
     def delete(self, loc):
@@ -769,16 +700,12 @@ class Library(object):
         tr = self.collection.get_track_by_loc(loc)
         if tr:
             self.collection.remove(tr)
-            path = common.local_file_from_url(tr.get_loc_for_io())
-            try:
-                os.unlink(path)
-            except OSError: # file not found?
-                traceback.print_exc()
-                pass
-            except:
-                common.log_exception(logger)
+            loc = tr.get_loc_for_io()
+            file = gio.File(loc)
+            if not file.delete():
+                logger.warning("Could not delete file %s." % loc)
 
-    # the below are not essential for 0.3.0, should be implemented only 
+    # the below are not essential for 0.3.0, should be implemented only
     # if time allows for it
 
     def set_layout(self, layout, default="Unknown"):
@@ -836,18 +763,18 @@ class TransferQueue(object):
         self.transferring = True
         self.current_pos += 1
         try:
-            while self.current_pos + 1 < len(self.queue) and not self._stop:
+            while self.current_pos  < len(self.queue) and not self._stop:
                 track = self.queue[self.current_pos]
-                loc = track.get_loc()
+                loc = track.get_loc_for_io()
                 self.library.add(loc)
 
                 # TODO: make this be based on filesize not count
                 progress = self.current_pos * 100 / len(self.queue)
-                print progress
                 event.log_event('track_transfer_progress', self, progress)
 
                 self.current_pos += 1
         finally:
+            self.queue = []
             self.transferring = False
             self.current_pos = -1
             self._stop = False
