@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen 
+# Copyright (C) 2008-2009 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,45 +15,38 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 #
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
+# The developers of the Exaile media player hereby grant permission
+# for non-GPL compatible GStreamer and Exaile plugins to be used and
+# distributed together with GStreamer and Exaile. This permission is
+# above and beyond the permissions granted by the GPL license by which
+# Exaile is covered. If you modify this code, you may extend this
+# exception to your version of the code, but you are not obligated to
+# do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
 # Here's where it all begins.....
 #
-# Holds the main Exaile class, whose instantiation starts up the entiriety 
+# Holds the main Exaile class, whose instantiation starts up the entiriety
 # of Exaile and which also handles Exaile shutdown.
 #
 # Also takes care of parsing commandline options.
 
-__version__ = '0.3.0.1'
-
-import locale, gettext
-
-# set the locale to LANG, or the user's default
-locale.setlocale(locale.LC_ALL, '')
-
-# this installs _ into python's global namespace, so we don't have to
-# explicitly import it elsewhere
-#TODO: make this work
-#gettext.install("exaile")
+import os
+import sys
+import logging
+import logging.handlers
 
 from xl.nls import gettext as _
-
 from xl import common, xdg, event
-import os, sys, logging, logging.handlers, time
+
+from xl.version import __version__
 
 # initiate the logger. logger params are set later
 logger = logging.getLogger(__name__)
 
 class Exaile(object):
     _exaile = None
-    
+
     def __init__(self):
         """
             Initializes Exaile.
@@ -73,27 +66,32 @@ class Exaile(object):
         if self.options.EventFilter:
             event.EVENT_MANAGER.logger_filter = self.options.EventFilter
 
-        #set up logging
+        # set up logging
         self.setup_logging()
 
-        #initial mainloop setup. The actual loop is started later, if necessary
+        # initial mainloop setup. The actual loop is started later,
+        # if necessary
         self.mainloop_init()
-        
+
         #initialize DbusManager
         if self.options.StartGui and self.options.Dbus:
             from xl import xldbus
             if xldbus.check_exit(self.options, self.args):
                 sys.exit(0)
             self.dbus = xldbus.DbusManager(self)
-        
+
         #load the rest.
         self.__init()
 
+        #connect dbus signals
+        if self.options.Dbus:
+            self.dbus._connect_signals()
+
         # On SIGTERM, quit normally.
         import signal
         signal.signal(signal.SIGTERM, (lambda sig, stack: self.quit()))
 
-        #run the GUIs mainloop, if needed
+        # run the GUIs mainloop, if needed
         if self.options.StartGui:
             import xlgui
             xlgui.mainloop()
@@ -102,6 +100,7 @@ class Exaile(object):
         """
             Initializes Exaile
         """
+        # pylint: disable-msg=W0201
         logger.info("Loading Exaile %s..." % __version__)
 
         logger.info("Loading settings...")
@@ -110,7 +109,7 @@ class Exaile(object):
         except common.VersionError:
             common.log_exception(log=logger)
             exit(1)
-        
+
         # Splash screen
         if self.options.StartGui:
             self.__show_splash()
@@ -126,7 +125,7 @@ class Exaile(object):
                 migrator.migrate(force=self.options.ForceImport)
                 del migrator
             except:
-                common.log_exception(log=logger, 
+                common.log_exception(log=logger,
                         message=_("Failed to migrate from 0.2.14"))
 
         # Initialize plugin manager
@@ -153,8 +152,8 @@ class Exaile(object):
         from xl import player
         from xl.player import queue
         self.player = player.get_player()()
-        self.queue = player.queue.PlayQueue(self.player, 
-                location=os.path.join(xdg.get_data_dirs()[0], 'queue.state') )
+        self.queue = queue.PlayQueue(self.player,
+                location=os.path.join(xdg.get_data_dirs()[0], 'queue.state'))
         event.log_event("player_loaded", self, None)
 
         # Initalize playlist manager
@@ -163,7 +162,7 @@ class Exaile(object):
         self.smart_playlists = playlist.PlaylistManager('smart_playlists',
             playlist.SmartPlaylist)
         if firstrun:
-            self._add_default_playlists() 
+            self._add_default_playlists()
         event.log_event("playlists_loaded", self, None)
 
         # Initialize dynamic playlist support
@@ -187,7 +186,7 @@ class Exaile(object):
         # Cover manager
         from xl import cover
         self.covers = cover.CoverManager(
-                cache_dir=os.path.join(xdg.get_data_dirs()[0], "covers"))
+                location=os.path.join(xdg.get_data_dirs()[0], "covers"))
 
         # Radio Manager
         from xl import radio
@@ -214,8 +213,8 @@ class Exaile(object):
                 gobject.idle_add(self.splash.destroy)
             event.log_event("gui_loaded", self, None)
 
-        self.queue._restore_player_state(
-                os.path.join(xdg.get_data_dirs()[0], 'player.state') )
+
+        restore = True
 
         if self.gui:
             # Find out if the user just passed in a list of songs
@@ -223,16 +222,22 @@ class Exaile(object):
             # using arg[2:] because arg[1:] will include --startgui
             args = [ os.path.abspath(arg) for arg in self.args ]
             if len(args) > 0:
+                restore = False
                 self.gui.open_uri(args[0], play=True)
             for arg in args[1:]:
                 self.gui.open_uri(arg)
 
+        if restore:
+            self.queue._restore_player_state(
+                    os.path.join(xdg.get_data_dirs()[0], 'player.state'))
+
         if firstrun:
             settings.set_option("general/first_run", False)
 
         self.loading = False
         Exaile._exaile = self
         event.log_event("exaile_loaded", self, None)
+        # pylint: enable-msg=W0201
 
     def __show_splash(self):
         """
@@ -251,23 +256,41 @@ class Exaile(object):
             console_format += " (%(name)s)" # add module name in debug mode
         elif self.options.Quiet:
             loglevel = logging.WARNING
-        # logfile level should always be INFO or higher
+        # Logfile level should always be INFO or higher
         if self.options.Quiet:
             logfilelevel = logging.INFO
         else:
             logfilelevel = loglevel
 
-        # logging to terminal
+        # Logging to terminal
         logging.basicConfig(level=loglevel, format=console_format)
 
-        # logging to file. this also automatically rotates the logs
+        # Create log directory
+        logdir = os.path.join(xdg.get_data_dir(), 'logs')
+        if not os.path.exists(logdir):
+            os.makedirs(logdir)
+
+        # Try to migrate logs from old location
+        from glob import glob
+        logfiles = glob(os.path.join(xdg.get_config_dir(), 'exaile.log*'))
+        for logfile in logfiles:
+            try:
+                # Try to move to new location
+                os.rename(logfile, os.path.join(logdir,
+                    os.path.basename(logfile)))
+            except OSError:
+                # Give up and simply remove
+                os.remove(logfile)
+
+        # Logging to file; this also automatically rotates the logs
         logfile = logging.handlers.RotatingFileHandler(
-                os.path.join(xdg.get_config_dir(), "exaile.log"),
+                os.path.join(logdir, 'exaile.log'),
                 mode='a', backupCount=5)
         logfile.doRollover() # each session gets its own file
         logfile.setLevel(logfilelevel)
-        formatter = logging.Formatter('%(asctime)s %(levelname)-8s: %(message)s (%(name)s)', 
-            datefmt="%m-%d %H:%M")
+        formatter = logging.Formatter(
+                '%(asctime)s %(levelname)-8s: %(message)s (%(name)s)',
+                datefmt="%m-%d %H:%M")
         logfile.setFormatter(formatter)
         logging.getLogger("").addHandler(logfile)
 
@@ -275,97 +298,114 @@ class Exaile(object):
         """
             Get the options for exaile
         """
-        from optparse import OptionParser
+        from optparse import OptionParser, OptionGroup
         usage = "Usage: %prog [option...|uri]"
         p = OptionParser(usage=usage)
-        
-        # Playback options
-        p.add_option("-n", "--next", dest="Next", action="store_true",
-            default=False, help=_("Play the next track"))
-        p.add_option("-p", "--prev", dest="Prev", action="store_true",
-            default=False,   help=_("Play the previous track"))
-        p.add_option("-s", "--stop", dest="Stop", action="store_true",
-            default=False, help=_("Stop playback"))
-        p.add_option("-a", "--play", dest="Play", action="store_true",
-            default=False, help=_("Play"))
-        p.add_option("-t", "--play-pause", dest="PlayPause", 
-            action="store_true", default=False, help=_("Toggle Play or Pause"))
-        p.add_option("--stop-after-current", dest="StopAfterCurrent",
-            action="store_true", default=False,
-            help=_("Stop playback after current track"))
 
-        # Current song options
-        p.add_option("-q", "--query", dest="Query", action="store_true",
-            default=False, help=_("Query player"))
-        p.add_option("--gui-query", dest="GuiQuery", action="store_true",
-            default=False, help=_("Show a popup of the currently playing track"))
-        p.add_option("--get-title", dest="GetTitle", action="store_true",
-            default=False, help=_("Print the title of current track"))
-        p.add_option("--get-album", dest="GetAlbum", action="store_true",
-            default=False, help=_("Print the album of current track"))
-        p.add_option("--get-artist", dest="GetArtist", action="store_true",
-            default=False, help=_("Print the artist of current track"))
-        p.add_option("--get-length", dest="GetLength", action="store_true",
-            default=False, help=_("Print the length of current track"))
-        p.add_option('--set-rating', dest="SetRating", action='store',
-            type='int', metavar='RATING', help=_('Set rating for current song'))
-        p.add_option('--get-rating', dest='GetRating', action='store_true',
-            default=False, help=_('Get rating for current song'))
-        p.add_option("--current-position", dest="CurrentPosition", 
-            action="store_true", default=False, 
-            help=_("Print the position inside the current track as time"))
-        p.add_option("--current-progress", dest="CurrentProgress",
-            action="store_true", default=False,
-            help=_("Print the progress inside the current track as percentage"))
+        group = OptionGroup(p, _('Playback options'))
+        group.add_option("-n", "--next", dest="Next", action="store_true",
+                default=False, help=_("Play the next track"))
+        group.add_option("-p", "--prev", dest="Prev", action="store_true",
+                default=False,   help=_("Play the previous track"))
+        group.add_option("-s", "--stop", dest="Stop", action="store_true",
+                default=False, help=_("Stop playback"))
+        group.add_option("-a", "--play", dest="Play", action="store_true",
+                default=False, help=_("Play"))
+        group.add_option("-t", "--play-pause", dest="PlayPause",
+                action="store_true", default=False,
+                help=_("Toggle Play or Pause"))
+        group.add_option("--stop-after-current", dest="StopAfterCurrent",
+                action="store_true", default=False,
+                help=_("Stop playback after current track"))
+        p.add_option_group(group)
 
-        # Volume options
-        p.add_option("-i", "--increase-vol", dest="IncreaseVolume", action="store", 
-            type="int", metavar="VOL", help=_("Increases the volume by VOL%"))
-        p.add_option("-l", "--decrease-vol", dest="DecreaseVolume", action="store",
-            type="int", metavar="VOL", help=_("Decreases the volume by VOL%"))
-        p.add_option("--get-volume", dest="GetVolume", action="store_true",
-            default=False, help=_("Print the current volume percentage"))
+        group = OptionGroup(p, _('Track options'))
+        group.add_option("-q", "--query", dest="Query", action="store_true",
+                default=False, help=_("Query player"))
+        group.add_option("--gui-query", dest="GuiQuery",
+                action="store_true", default=False,
+                help=_("Show a popup of the currently playing track"))
+        group.add_option("--get-title", dest="GetTitle", action="store_true",
+                default=False, help=_("Print the title of current track"))
+        group.add_option("--get-album", dest="GetAlbum", action="store_true",
+                default=False, help=_("Print the album of current track"))
+        group.add_option("--get-artist", dest="GetArtist", action="store_true",
+                default=False, help=_("Print the artist of current track"))
+        group.add_option("--get-length", dest="GetLength", action="store_true",
+                default=False, help=_("Print the length of current track"))
+        group.add_option('--set-rating', dest="SetRating",
+                action='store', type='int', metavar='RATING',
+                help=_('Set rating for current song'))
+        group.add_option('--get-rating', dest='GetRating', action='store_true',
+                default=False, help=_('Get rating for current song'))
+        group.add_option("--current-position", dest="CurrentPosition",
+                action="store_true", default=False,
+                help=_("Print the position inside the current track as time"))
+        group.add_option("--current-progress", dest="CurrentProgress",
+                action="store_true", default=False, help=_("Print the "
+                "progress inside the current track as percentage"))
+        p.add_option_group(group)
 
-        # Other options
-        p.add_option("--new", dest="NewInstance", action="store_true",
-            default=False, help=_("Start new instance"))
-        p.add_option("--version", dest="ShowVersion", action="store_true")
-        p.add_option("--start-minimized", dest="StartMinimized", action="store_true",
-            default=False, help=_("Start minimized (to tray, if possible)"))
-        p.add_option("--safemode", dest="SafeMode", action="store_true",
-            default=False, help=_("Start in safe mode - sometimes useful "
-            "when you're running into problems"))
-        p.add_option("--force-import", dest="ForceImport", 
-            action="store_true", default=False, help=_("Force import of "
-            "old data from 0.2.x. Overwrites current data."))
-        p.add_option("--no-import", dest="NoImport", 
-            action="store_true", default=False, help=_("Do not import "
-            "old data from 0.2.x."))
+        group = OptionGroup(p, _('Volume options'))
+        group.add_option("-i", "--increase-vol", dest="IncreaseVolume",
+                action="store", type="int", metavar="VOL",
+                help=_("Increases the volume by VOL%"))
+        group.add_option("-l", "--decrease-vol", dest="DecreaseVolume",
+                action="store", type="int", metavar="VOL",
+                help=_("Decreases the volume by VOL%"))
+        group.add_option("--get-volume", dest="GetVolume", action="store_true",
+                default=False, help=_("Print the current volume percentage"))
+        p.add_option_group(group)
+
+        group = OptionGroup(p, _('Other options'))
+        group.add_option("--new", dest="NewInstance", action="store_true",
+                default=False, help=_("Start new instance"))
+        group.add_option("--version", dest="ShowVersion", action="store_true")
+        group.add_option("--start-minimized", dest="StartMinimized",
+                action="store_true", default=False,
+                help=_("Start minimized (to tray, if possible)"))
+        group.add_option("--toggle-visible", dest="GuiToggleVisible",
+                action="store_true", default=False,
+                help=_("Toggle visibility of the GUI (if possible)"))
+        group.add_option("--safemode", dest="SafeMode", action="store_true",
+                default=False, help=_("Start in safe mode - sometimes "
+                "useful when you're running into problems"))
+        group.add_option("--force-import", dest="ForceImport",
+                action="store_true", default=False, help=_("Force import of"
+                " old data from 0.2.x. Overwrites current data."))
+        group.add_option("--no-import", dest="NoImport",
+                action="store_true", default=False, help=_("Do not import "
+                "old data from 0.2.x."))
+        p.add_option_group(group)
 
         # development and debug options
-        p.add_option("--datadir", dest="UseDataDir", help=_("Set data directory"))
-        p.add_option("--debug", dest="Debug", action="store_true",
-            default=False, help=_("Show debugging output"))
-        p.add_option("--eventdebug", dest="DebugEvent", 
-            action="store_true", default=False, 
-            help=_("Enable debugging of xl.event. Generates LOTS of output"))
-        p.add_option("--eventfilter", dest="EventFilter",
-            action='store', type='string', help=_("Filter event debug "
-            "output"))
-        p.add_option("--quiet", dest="Quiet", action="store_true",
-            default=False, help=_("Reduce level of output"))
-        p.add_option('--startgui', dest='StartGui', action='store_true',
-            default=False)
-        p.add_option('--no-dbus', dest='Dbus', action='store_false',
-            default=True, help=_("Disable D-Bus support"))
-        p.add_option('--no-hal', dest='Hal', action='store_false',
-            default=True, help=_("Disable HAL support."))
+        group = OptionGroup(p, _('Development/Debug options'))
+        group.add_option("--datadir", dest="UseDataDir",
+                help=_("Set data directory"))
+        group.add_option("--debug", dest="Debug", action="store_true",
+                default=False, help=_("Show debugging output"))
+        group.add_option("--eventdebug", dest="DebugEvent",
+                action="store_true", default=False, help=_("Enable debugging"
+                " of xl.event. Generates LOTS of output"))
+        group.add_option("--eventfilter", dest="EventFilter",
+                action='store', type='string', help=_("Filter event debug "
+                "output"))
+        group.add_option("--quiet", dest="Quiet", action="store_true",
+                default=False, help=_("Reduce level of output"))
+        group.add_option('--startgui', dest='StartGui', action='store_true',
+                default=False)
+        group.add_option('--no-dbus', dest='Dbus', action='store_false',
+                default=True, help=_("Disable D-Bus support"))
+        group.add_option('--no-hal', dest='Hal', action='store_false',
+                default=True, help=_("Disable HAL support."))
+        p.add_option_group(group)
+
         return p
 
     def version(self):
         print r"""   ____          _ __    __
   / __/_ _____ _(_) /__ / /
- / _/ \ \ / _ `/ / / -_)_/ 
+ / _/ \ \ / _ `/ / / -_)_/
 /___//_\_\\_,_/_/_/\__(_)   v%s
 """%__version__
         exit()
@@ -377,7 +417,7 @@ class Exaile(object):
         from xl import playlist
         # entire playlist
         entire_lib = playlist.SmartPlaylist(_("Entire Library"),
-            collection=self.collection) 
+            collection=self.collection)
         self.smart_playlists.save_playlist(entire_lib, overwrite=True)
 
         # random playlists
@@ -390,7 +430,7 @@ class Exaile(object):
 
         # rating based playlists
         for item in (3, 4):
-            pl = playlist.SmartPlaylist(_("Rating > %d") % item, 
+            pl = playlist.SmartPlaylist(_("Rating > %d") % item,
                 collection=self.collection)
             pl.add_param('__rating', '>', item)
             self.smart_playlists.save_playlist(pl, overwrite=True)
@@ -400,7 +440,7 @@ class Exaile(object):
         gobject.threads_init()
         if self.options.Dbus:
             import dbus, dbus.mainloop.glib
-            dbus_loop = dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
+            dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
             dbus.mainloop.glib.threads_init()
             dbus.mainloop.glib.gthreads_init()
         if self.options.StartGui:
@@ -431,7 +471,7 @@ class Exaile(object):
 
             takes care of saving prefs, databases, etc.
         """
-        if self.quitting: 
+        if self.quitting:
             return
         self.quitting = True
         logger.info("Exaile is shutting down...")
@@ -444,14 +484,14 @@ class Exaile(object):
                 except:
                     pass
 
-        # stop the various idle based threads so they don't freak out when the
-        # program exits.  Silly Python.
+        # stop the various idle based threads so they don't freak out
+        # when the program exits.  Silly Python.
         event.IDLE_MANAGER.stop()
         for timer in event._TIMERS:
             timer.cancel()
 
         # this event should be used by plugins and modules that dont need
-        # to be saved in any particular order. modules that might be 
+        # to be saved in any particular order. modules that might be
         # touched by events triggered here should be added statically
         # below.
         event.log_event("quit_application", self, None, async=False)
@@ -462,11 +502,11 @@ class Exaile(object):
         if self.gui:
             self.gui.quit()
 
-        self.covers.save_cover_db()
+        self.covers.save()
 
         self.collection.save_to_location()
-        
-        #Save order of custom playlists
+
+        # Save order of custom playlists
         self.playlists.save_order()
         self.stations.save_order()
 
