@@ -1,6 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen 
-#
-# Copyright (C) 2008-2009 Adam Olsen 
+# Copyright (C) 2008-2009 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,35 +15,40 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 #
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
-# from your version.
-#
-#
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
+# The developers of the Exaile media player hereby grant permission
+# for non-GPL compatible GStreamer and Exaile plugins to be used and
+# distributed together with GStreamer and Exaile. This permission is
+# above and beyond the permissions granted by the GPL license by which
+# Exaile is covered. If you modify this code, you may extend this
+# exception to your version of the code, but you are not obligated to
+# do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
 __all__ = ['main', 'panel', 'playlist']
 
-from xl.nls import gettext as _
-import gtk, gtk.glade, gobject, logging, os, urlparse
-from xl import xdg, common, event, metadata, settings, playlist as _xpl
+import logging
+import os
+import urlparse
 
-from xlgui import commondialogs, cover 
+import gobject
+import gtk
+
+from xl.nls import gettext as _
+logger = logging.getLogger(__name__)
+from xl import xdg, common, event, metadata, settings, playlist as _xpl
+try:
+    import gtk.glade
+    gtk.glade.textdomain('exaile')
+    if xdg.local_hack:
+        gtk.glade.bindtextdomain('exaile', os.path.join(xdg.exaile_dir, 'po'))
+except ImportError:
+    logger.warning(
+        "Failed to import gtk.glade, interface "
+        "will not be fully translated.")
+
+from xlgui import commondialogs, cover
 from xlgui import devices, guiutil, icons, prefs, queue
 
-logger = logging.getLogger(__name__)
 
 ###
 # Set up xl/event to work with the gtk event loop
@@ -65,12 +68,11 @@ class Main(object):
     """
     _main = None
     def __init__(self, exaile):
-        """ 
+        """
             Initializes the GUI
 
             @param exaile: The Exaile instance
         """
-        import xl.main as xlmain
         from xlgui import main, panel, tray, progress
         from xlgui.panel import collection, radio, playlists, files
 
@@ -78,9 +80,9 @@ class Main(object):
         self.first_removed = False
         self.tray_icon = None
         self.panels = {}
-        self.xml = gtk.glade.XML(xdg.get_data_path("glade/main.glade"),
-            'ExaileWindow', 'exaile')
-        self.progress_box = self.xml.get_widget('progress_box')
+        self.builder = gtk.Builder()
+        self.builder.add_from_file(xdg.get_data_path("ui/main.ui"))
+        self.progress_box = self.builder.get_object('progress_box')
         self.progress_manager = progress.ProgressManager(self.progress_box)
 
         self.icons = icons.IconManager()
@@ -98,89 +100,46 @@ class Main(object):
                 xdg.get_data_path('images'))
 
         logger.info("Loading main window...")
-        self.main = main.MainWindow(self, self.xml,
+        self.main = main.MainWindow(self, self.builder,
             exaile.collection, exaile.player, exaile.queue, exaile.covers)
-        self.panel_notebook = self.xml.get_widget('panel_notebook')
-        self.play_toolbar = self.xml.get_widget('play_toolbar')
+        self.panel_notebook = self.builder.get_object('panel_notebook')
+        self.play_toolbar = self.builder.get_object('play_toolbar')
 
         logger.info("Loading panels...")
-        self.last_selected_panel = settings.get_option(
-            'gui/last_selected_panel', None)
         self.panels['collection'] = collection.CollectionPanel(self.main.window,
             exaile.collection, _show_collection_empty_message=True)
-        self.panels['radio'] = radio.RadioPanel(self.main.window, exaile.collection, 
+        self.panels['radio'] = radio.RadioPanel(self.main.window, exaile.collection,
             exaile.radio, exaile.stations)
-        self.panels['playlists'] = playlists.PlaylistsPanel(self.main.window, 
+        self.panels['playlists'] = playlists.PlaylistsPanel(self.main.window,
             exaile.playlists, exaile.smart_playlists, exaile.collection)
         self.panels['files'] = files.FilesPanel(self.main.window, exaile.collection)
 
         for panel in ('collection', 'radio', 'playlists', 'files'):
             self.add_panel(*self.panels[panel].get_panel())
 
-        try:
-            selected_panel = self.panels[self.last_selected_panel]._child
-            selected_panel_num = self.panel_notebook.page_num(selected_panel)
-            self.panel_notebook.set_current_page(selected_panel_num)
-        except KeyError:
-            pass
-
         # add the device panels
         for device in self.exaile.devices.list_devices():
             if device.connected:
                 self.add_device_panel(None, None, device)
-       
+
         logger.info("Connecting panel events...")
         self.main._connect_panel_events()
 
-        logger.info("Connecting main window events...")
-        self._connect_events()
-
         if settings.get_option('gui/use_tray', False):
             self.tray_icon = tray.TrayIcon(self.main)
 
-        event.add_callback(self._on_quit_application, 'quit_application')
-
         self.device_panels = {}
         event.add_callback(self.add_device_panel, 'device_connected')
         event.add_callback(self.remove_device_panel, 'device_disconnected')
+        event.add_callback(self.on_gui_loaded, 'gui_loaded')
 
         logger.info("Done loading main window...")
         Main._main = self
 
-    def _connect_events(self):
-        """
-            Connects the various events to their handlers
-        """
-        self.xml.signal_autoconnect({
-            'on_about_item_activate': self.show_about_dialog,
-            'on_scan_collection_item_activate': self.on_rescan_collection,
-            'on_randomize_playlist_item_activate': self.on_randomize_playlist,
-            'on_collection_manager_item_activate': self.collection_manager,
-            'on_goto_playing_track_activate': self.on_goto_playing_track,
-            'on_queue_manager_item_activate': self.queue_manager,
-            'on_preferences_item_activate': lambda *e: self.show_preferences(),
-            'on_device_manager_item_activate': lambda *e: self.show_devices(),
-            'on_cover_manager_item_activate': self.show_cover_manager,
-            'on_open_item_activate': self.open_dialog,
-            'on_open_url_item_activate': self.open_url,
-            'on_export_current_playlist_activate': self.export_current_playlist,
-            'on_panel_notebook_switch_page': self.on_panel_switch,
-            'on_track_properties_activate':self.on_track_properties,
-            'on_clear_playlist_item_activate': self.main.on_clear_playlist,
-        })
-
-    def _on_quit_application(self, event, sender, data):
-        """
-            Updates settings affected by GUI interaction
-        """
-        if not self.last_selected_panel:
-            self.last_selected_panel = 0
-        settings.set_option('gui/last_selected_panel', self.last_selected_panel)
-        
     def export_current_playlist(self, *e):
         pl = self.main.get_current_playlist ().playlist
         name = pl.get_name() + ".m3u"
-        
+
         dialog = commondialogs.FileOperationDialog(_("Export current playlist..."),
             None, gtk.FILE_CHOOSER_ACTION_SAVE,
             buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
@@ -206,7 +165,7 @@ class Main(object):
                 except _xpl.InvalidPlaylistTypeException:
                     commondialogs.error(None, _('Invalid file extension, file not saved'))
         dialog.destroy()
-        
+
     def open_url(self, *e):
         """
             Displays a dialog to open a url
@@ -215,7 +174,7 @@ class Main(object):
         _('Open URL'))
         dialog.set_transient_for(self.main.window)
         dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
-        
+
         result = dialog.run()
         dialog.hide()
         if result == gtk.RESPONSE_OK:
@@ -230,6 +189,8 @@ class Main(object):
             self.main.window, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OPEN, gtk.RESPONSE_OK))
         dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
+        dialog.set_local_only(False) # enable gio
+        dialog.set_select_multiple(True)
 
         supported_file_filter = gtk.FileFilter()
         supported_file_filter.set_name(_("Supported Files"))
@@ -240,9 +201,9 @@ class Main(object):
         all_file_filter = gtk.FileFilter()
         all_file_filter.set_name(_("All Files"))
 
-        for ext in metadata.SUPPORTED_MEDIA:
-            supported_file_filter.add_pattern('*' + ext)
-            audio_file_filter.add_pattern('*' + ext)
+        for ext in metadata.formats.keys():
+            supported_file_filter.add_pattern('*.' + ext)
+            audio_file_filter.add_pattern('*.' + ext)
 
         playlist_file_types = ('m3u', 'pls', 'asx', 'xspf')
         for playlist_file_type in playlist_file_types:
@@ -256,6 +217,25 @@ class Main(object):
         dialog.add_filter(playlist_file_filter)
         dialog.add_filter(all_file_filter)
 
+        result = dialog.run()
+        dialog.hide()
+        if result == gtk.RESPONSE_OK:
+            files = dialog.get_filenames()
+            for f in files:
+                self.open_uri(f, play=False)
+
+    def open_dir(self, *e):
+        """
+            Shows a dialog for opening (multiple) directories
+        """
+
+        dialog = gtk.FileChooserDialog(_("Choose a file to open"),
+            self.main.window, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                gtk.STOCK_OPEN, gtk.RESPONSE_OK))
+        dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
+        dialog.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
+        dialog.set_select_multiple(True)
+
         result = dialog.run()
         dialog.hide()
         if result == gtk.RESPONSE_OK:
@@ -274,7 +254,7 @@ class Main(object):
             Determines the type of a uri, imports it into a playlist, and
             starts playing it
         """
-        from xl import playlist, track
+        from xl import playlist, trax
         if playlist.is_valid_playlist(uri):
             pl = playlist.import_playlist(uri)
             self.main.add_playlist(pl)
@@ -283,8 +263,10 @@ class Main(object):
         else:
             pl = self.main.get_selected_playlist()
             column, descending = pl.get_sort_by()
-            tracks = track.get_tracks_from_uri(uri)
-            tracks.sort(key=lambda track: track.sort_param(column), reverse=descending)
+
+            tracks = trax.get_tracks_from_uri(uri)
+            tracks = trax.sort_tracks(pl.return_order_tags(column), tracks)
+
             try:
                 pl.playlist.add_tracks(tracks)
                 pl.playlist.set_current_pos(len(pl.playlist) - len(tracks))
@@ -345,6 +327,21 @@ class Main(object):
             coll.thaw_libraries()
             self.on_rescan_collection()
 
+    def on_gui_loaded(self, event, object, nothing):
+        """
+            Allows plugins to be the last selected panel
+        """
+        try:
+            last_selected_panel = settings.get_option(
+                'gui/last_selected_panel', 'collection')
+            panel = self.panels[last_selected_panel]._child
+            panel_num = self.panel_notebook.page_num(panel)
+            self.panel_notebook.set_current_page(panel_num)
+            # Fix track info not displaying properly when resuming after a restart.
+            self.main._update_track_information()
+        except KeyError:
+            pass
+
     def on_goto_playing_track(self, *e):
         track = self.exaile.queue.get_current()
         pl = self.main.get_current_playlist()
@@ -353,30 +350,31 @@ class Main(object):
             pl.list.scroll_to_cell(index)
             pl.list.set_cursor(index)
         #TODO implement a way to browse through all playlists and search for the track
-            
+
     def on_rescan_collection(self, *e):
         """
             Called when the user wishes to rescan the collection
         """
-        from xlgui import collection as guicol
-        thread = guicol.CollectionScanThread(self, self.exaile.collection, 
-                self.panels['collection'])
-        self.progress_manager.add_monitor(thread,
-            _("Scanning collection..."), 'gtk-refresh')
+        if not self.exaile.collection._scanning:
+            from xlgui.collection import CollectionScanThread
+            thread = CollectionScanThread(self, self.exaile.collection,
+                    self.panels['collection'])
+            self.progress_manager.add_monitor(thread,
+                _("Scanning collection..."), 'gtk-refresh')
 
     def on_randomize_playlist(self, *e):
         pl = self.main.get_selected_playlist()
         pl.playlist.randomize()
         pl._set_tracks(pl.playlist.get_tracks())
         pl.reorder_songs()
-    
+
     def on_track_properties(self, *e):
         pl = self.main.get_selected_playlist()
         if not pl.properties_dialog():
             if self.exaile.player.current:
+                from xlgui import properties
                 dialog = properties.TrackPropertiesDialog(self.main.window,
-                        self.exaile.player.current)
-                result = dialog.run()
+                        [self.exaile.player.current])
                 dialog.hide()
 
 
@@ -393,6 +391,7 @@ class Main(object):
 
             # the first tab in the panel is a stub that just stops libglade from
             # complaining
+            # TODO: Check if this is valid for GtkBuilder
             self.panel_notebook.remove_page(0)
 
     def remove_panel(self, child):
@@ -403,19 +402,26 @@ class Main(object):
         raise ValueError("No such panel")
 
     def on_panel_switch(self, notebook, page, pagenum):
+        """
+            Saves the currently selected panel
+        """
+        if self.exaile.loading:
+            return
+
         page = notebook.get_nth_page(pagenum)
-        for id, panel in self.panels.items():
+        for i, panel in self.panels.items():
             if panel._child == page:
-                self.last_selected_panel = id
+                settings.set_option('gui/last_selected_panel', i)
+                return
 
     def show_about_dialog(self, *e):
         """
             Displays the about dialog
         """
         import xl.main as xlmain
-        xml = gtk.glade.XML(xdg.get_data_path('glade/about_dialog.glade'),
-            'AboutDialog', 'exaile')
-        dialog = xml.get_widget('AboutDialog')
+        builder = gtk.Builder()
+        builder.add_from_file(xdg.get_data_path('ui/about_dialog.ui'))
+        dialog = builder.get_object('AboutDialog')
         logo = gtk.gdk.pixbuf_new_from_file(
             xdg.get_data_path('images/exailelogo.png'))
         dialog.set_logo(logo)
@@ -424,7 +430,7 @@ class Main(object):
         dialog.set_transient_for(self.main.window)
         dialog.connect('response', lambda d, r: d.destroy())
         dialog.show()
-        
+
     def quit(self):
         """
             Quits the gui, saving anything that needs to be saved
@@ -445,7 +451,7 @@ class Main(object):
             elif issubclass(device.panel_type, xlgui.panel.Panel):
                 paneltype = device.panel_type
 
-        panel = paneltype(self.main.window, self.main, 
+        panel = paneltype(self.main.window, self.main,
             device, device.get_name())
 
         sort = True
@@ -456,7 +462,7 @@ class Main(object):
 
         self.device_panels[device.get_name()] = panel
         gobject.idle_add(self.add_panel, *panel.get_panel())
-        thread = collection.CollectionScanThread(self.main, 
+        thread = collection.CollectionScanThread(self.main,
                 device.get_collection(), panel)
         self.progress_manager.add_monitor(thread,
                 _("Scanning %s..."%device.name), 'gtk-refresh')
@@ -479,10 +485,10 @@ def show_splash(show=True):
     if not show: return
     image = gtk.Image()
     image.set_from_file(xdg.get_data_path("images/splash.png"))
-    xml = gtk.glade.XML(xdg.get_data_path("glade/splash.glade"), 'SplashScreen',
-        'exaile')
-    splash_screen = xml.get_widget('SplashScreen')
-    box = xml.get_widget('splash_box')
+    builder = gtk.Builder()
+    builder.add_from_file(xdg.get_data_path("ui/splash.ui"))
+    splash_screen = builder.get_object('SplashScreen')
+    box = builder.get_object('splash_box')
     box.pack_start(image, True, True)
     splash_screen.set_transient_for(None)
     splash_screen.show_all()
