@@ -1,6 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen 
-#
-# Copyright (C) 2008-2009 Adam Olsen 
+# Copyright (C) 2008-2009 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,36 +15,30 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 #
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
-# from your version.
-#
-#
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
+# The developers of the Exaile media player hereby grant permission
+# for non-GPL compatible GStreamer and Exaile plugins to be used and
+# distributed together with GStreamer and Exaile. This permission is
+# above and beyond the permissions granted by the GPL license by which
+# Exaile is covered. If you modify this code, you may extend this
+# exception to your version of the code, but you are not obligated to
+# do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import gtk, os.path, urllib, time
-import gtk.gdk, pango, gobject
-from xl import xdg, track, playlist, common, settings, event
-from xl.nls import gettext as _
+import os.path
 import threading
-from xlgui import rating
+import time
+import urllib
+from urllib2 import urlparse
 
-try:
-    import sexy
-except ImportError:
-    sexy = None
+import gio
+import gobject
+import gtk
+import gtk.gdk
+import pango
+
+from xl import xdg, playlist, common, settings, event, trax
+from xl.nls import gettext as _
+from xlgui import rating
 
 def _idle_callback(func, callback, *args, **kwargs):
     value = func(*args, **kwargs)
@@ -62,7 +54,7 @@ def idle_add(callback=None):
         from the function that calls a function with this decorator.  Instead,
         you must use the callback parameter.  If the wrapped function returns
         a value, it will be passed in as a parameter to the callback function.
-        
+
         @param callback: optional callback that will be called when the
             wrapped function is done running
     """
@@ -70,7 +62,7 @@ def idle_add(callback=None):
         def wrapped(*args, **kwargs):
             gobject.idle_add(_idle_callback, f, callback,
                 *args, **kwargs)
-        
+
         return wrapped
     return wrap
 
@@ -101,6 +93,15 @@ def gtkrun(f):
 
     return wrapper
 
+def get_workarea_size():
+    """
+        Returns the height and width of the work area
+    """
+    rootwindow = gtk.gdk.get_default_root_window()
+    workarea = gtk.gdk.atom_intern('_NET_WORKAREA')
+
+    return rootwindow.property_get(workarea)[2][2:4] # W,H
+
 class ScalableImageWidget(gtk.Image):
     """
         Custom resizeable image widget
@@ -122,8 +123,8 @@ class ScalableImageWidget(gtk.Image):
         """
             Sets the image
         """
-        self.loc = image
-        self.pixbuf = gtk.gdk.pixbuf_new_from_file(image)
+        self.loc = gio.File(image).get_path()
+        self.pixbuf = gtk.gdk.pixbuf_new_from_file(self.loc)
 
         self._set_image(self.pixbuf, fill)
 
@@ -153,7 +154,9 @@ class DragTreeView(gtk.TreeView):
     """
         A TextView that does easy dragging/selecting/popup menu
     """
+    targets = [("text/uri-list", 0, 0)]
     dragged_data = dict()
+    
     def __init__(self, cont, receive=True, source=True):
         """
             Initializes the tree and sets up the various callbacks
@@ -161,18 +164,16 @@ class DragTreeView(gtk.TreeView):
         gtk.TreeView.__init__(self)
         self.cont = cont
 
-        self.targets = [("text/uri-list", 0, 0)]
-
         if source:
             self.drag_source_set(
                 gtk.gdk.BUTTON1_MASK, self.targets,
                 gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_MOVE)
 
         if receive:
-            self.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.targets, 
+            self.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.targets,
                 gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_DEFAULT|
                 gtk.gdk.ACTION_MOVE)
-            self.connect('drag_data_received', 
+            self.connect('drag_data_received',
                 self.cont.drag_data_received)
             self.connect('drag_data_delete',
                 self.cont.drag_data_delete)
@@ -192,7 +193,7 @@ class DragTreeView(gtk.TreeView):
         """
             Called when a button is released
         """
-        if event.button != 1 or self.dragging: 
+        if event.button != 1 or self.dragging:
             self.dragging = False
             return True
         if event.state & (gtk.gdk.SHIFT_MASK|gtk.gdk.CONTROL_MASK):
@@ -213,7 +214,7 @@ class DragTreeView(gtk.TreeView):
         """
         self.dragging = False
         self.unset_rows_drag_dest()
-        self.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.targets, 
+        self.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.targets,
             gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_MOVE)
 
     def drag_begin(self, list, context):
@@ -226,8 +227,8 @@ class DragTreeView(gtk.TreeView):
         selection = self.get_selection()
         if selection.count_selected_rows() > 1:
             self.drag_source_set_icon_stock('gtk-dnd-multiple')
-        else: self.drag_source_set_icon_stock('gtk-dnd')
-        return False
+        else:
+            self.drag_source_set_icon_stock('gtk-dnd')
 
     def drag_motion(self, treeview, context, x, y, timestamp):
         """
@@ -251,16 +252,16 @@ class DragTreeView(gtk.TreeView):
         x = int(x)
         y = int(y)
         path = self.get_path_at_pos(x, y)
-            
+
         if path:
-            if event.button != 3: 
+            if event.button != 3:
                 if event.type == gtk.gdk._2BUTTON_PRESS:
                     self.cont.button_press(button, event)
 
-                if selection.count_selected_rows() <= 1: 
+                if selection.count_selected_rows() <= 1:
                     return False
-                else: 
-                    if selection.path_is_selected(path[0]): 
+                else:
+                    if selection.path_is_selected(path[0]):
                         if event.state & (gtk.gdk.SHIFT_MASK|gtk.gdk.CONTROL_MASK):
                             selection.unselect_path(path[0])
                         return True
@@ -271,62 +272,70 @@ class DragTreeView(gtk.TreeView):
             if not selection.count_selected_rows():
                 selection.select_path(path[0])
         return self.cont.button_press(button, event)
-    
+
     #TODO maybe move this somewhere else? (along with _handle_unknown_drag_data)
     def get_drag_data(self, locs, compile_tracks = True, existing_tracks = []):
         """
             Handles the locations from drag data
-        
+
             @param locs: locations we are dealing with (can
                 be anything from a file to a folder)
             @param compile_tracks: if true any tracks in the playlists
                 that are not found as tracks are added to the list of tracks
             @param existing_tracks: a list of tracks that have already
-                been loaded from files (used to skip loading the dragged 
+                been loaded from files (used to skip loading the dragged
                 tracks from the filesystem)
-            
+
             @returns: a 2 tuple in which the first part is a list of tracks
                 and the second is a list of playlist (note: any files that are
                 in a playlist are not added to the list of tracks, but a track could
                 be both in as a found track and part of a playlist)
         """
         #TODO handle if they pass in existing tracks
-        tracks = []
+        trs = []
         playlists = []
         for loc in locs:
-            loc = urllib.unquote(loc)
             (found_tracks, found_playlist) = self._handle_unknown_drag_data(loc)
-            tracks.extend(found_tracks)
+            trs.extend(found_tracks)
             playlists.extend(found_playlist)
-            
+
         if compile_tracks:
             #Add any tracks in the playlist to the master list of tracks
             for playlist in playlists:
                 for track in playlist.get_tracks():
-                    if track not in tracks:
-                        tracks.append(track)
-                
-        return (tracks, playlists)
-    
+                    if track not in trs:
+                        trs.append(track)
+
+        return (trs, playlists)
+
     def _handle_unknown_drag_data(self, loc):
         """
             Handles unknown drag data that has been recieved by
             drag_data_received.  Unknown drag data is classified as
             any loc (location) that is not in the collection of tracks
             (i.e. a new song, or a new playlist)
-            
+
             @param loc:
                 the location of the unknown drag data
-            
+
             @returns: a 2 tuple in which the first part is a list of tracks
                 and the second is a list of playlist
         """
-        if loc in DragTreeView.dragged_data:
-            new_track = DragTreeView.dragged_data[loc]
-            del DragTreeView.dragged_data[loc]
-            return ([new_track],[])
-        elif track.is_valid_track(loc):
-            new_track = track.Track(loc)
+        filetype = None
+        info = urlparse.urlparse(loc)
+
+        # don't use gio to test the filetype if it's a non-local file
+        # (otherwise gio will try to connect to every remote url passed in and
+        # cause the gui to hang)
+        if info.scheme in ('file', ''):
+            try:
+                filetype = gio.File(loc).query_info(
+                    'standard::type').get_file_type()
+            except gio.Error:
+                filetype = None
+
+        if trax.is_valid_track(loc) or info.scheme not in ('file', ''):
+            new_track = trax.Track(loc)
             return ([new_track],[])
         elif playlist.is_valid_playlist(loc):
             #User is dragging a playlist into the playlist list
@@ -334,66 +343,29 @@ class DragTreeView(gtk.TreeView):
             # to the list
             new_playlist = playlist.import_playlist(loc)
             return ([], [new_playlist])
-        elif loc.startswith("file://") and os.path.isdir(loc[7:]):
-            #They dropped a folder
-            new_tracks = [] 
-            new_playlist = []
-            for root, dirs, files in os.walk(loc[7:]):
-                files.sort()
-                for file in files:
-                    full_path = os.path.join(root, file)
-                    (found_tracks, found_playlist) = self._handle_unknown_drag_data(full_path)
-                    new_tracks.extend(found_tracks)
-                    new_playlist.extend(found_playlist) 
-            return (new_tracks, new_playlist)
+        elif filetype == gio.FILE_TYPE_DIRECTORY:
+            return (trax.get_tracks_from_uri(loc), [])
         else: #We don't know what they dropped
             return ([], [])
 
-class EntryWithClearButton(object):
-    """
-        A gtk.Entry with a clear icon
-    """
-    def __init__(self):
-        """
-            Initializes the entry
-        """
-        if sexy:
-            self.entry = sexy.IconEntry()
-            image = gtk.Image()
-            image.set_from_stock('gtk-clear', gtk.ICON_SIZE_SMALL_TOOLBAR)
-            self.entry.set_icon(sexy.ICON_ENTRY_SECONDARY, image)
-            self.entry.connect('icon-released', self.icon_released)
-        else:
-            self.entry = gtk.Entry()
-
-    def icon_released(self, *e):
-        """
-            Called when the user clicks the entry icon
-        """
-        self.entry.set_text('')
-
-    def __getattr__(self, attr):
-        """
-            If this object doesn't have the attribute, check the gtk.Entry for
-            it
-        """
-        if attr == 'entry': return self.entry
-        return getattr(self.entry, attr)
-
-class SearchEntry(EntryWithClearButton):
+class SearchEntry(object):
     """
         A gtk.Entry that emits the "activated" signal when something has
         changed after the specified timeout
     """
-    def __init__(self, timeout=500):
+    def __init__(self, entry=None, timeout=500):
         """
             Initializes the entry
         """
-        EntryWithClearButton.__init__(self)
-        self.timeout = 500
+        self.entry = entry
+        self.timeout = timeout
         self.change_id = None
 
+        if self.entry is None:
+            self.entry = gtk.Entry()
+
         self.entry.connect('changed', self.on_entry_changed)
+        self.entry.connect('icon-press', self.on_entry_icon_press)
 
     def on_entry_changed(self, *e):
         """
@@ -405,11 +377,24 @@ class SearchEntry(EntryWithClearButton):
         self.change_id = gobject.timeout_add(self.timeout,
             self.entry_activate)
 
+    def on_entry_icon_press(self, entry, icon_pos, event):
+        """
+            Clears the entry
+        """
+        self.entry.set_text('')
+
     def entry_activate(self, *e):
         """
             Emit the activate signal
         """
-        self.entry.emit('activate')
+        self.entry.activate()
+
+    def __getattr__(self, attr):
+        """
+            Tries to pass attribute requests
+            to the internal entry item
+        """
+        return getattr(self.entry, attr)
 
 def get_icon(id, size=gtk.ICON_SIZE_BUTTON):
     """
@@ -421,19 +406,79 @@ def get_icon(id, size=gtk.ICON_SIZE_BUTTON):
         if icon: return icon
     except gobject.GError:
         pass
-    
+
     # If no stock icon exists for the specified ID, search in the "images" data
     # directory.
     path = xdg.get_data_path('images', id + '.png')
-    
+
     # Fallback to the "track.png" file.
     if not path:
         path = xdg.get_data_path('images', 'track.png')
-    
+
     return gtk.gdk.pixbuf_new_from_file(path)
 
+class MuteButton(object):
+    """
+        Allows for immediate muting of the volume and
+        indicates the current volume level via an icon
+    """
+    def __init__(self, button):
+        self.button = button
+        self.restore_volume = settings.get_option('player/volume', 1)
+        self.icon_names = ['low', 'medium', 'high']
+
+        self.button.connect('toggled', self.on_toggled)
+        event.add_callback(self.on_setting_change, 'player_option_set')
+
+    def update_volume_icon(self, volume):
+        """
+            Sets the volume level indicator
+        """
+        icon_name = 'audio-volume-muted'
+        tooltip = _('Muted')
+
+        if volume > 0:
+            i = int(round(volume * 2))
+            icon_name = 'audio-volume-%s' % self.icon_names[i]
+            #TRANSLATORS: Volume percentage
+            tooltip = '%d%%' % (volume * 100)
+
+        if volume == 1.0:
+            tooltip = _('Full Volume')
+
+        self.button.child.set_from_icon_name(icon_name, gtk.ICON_SIZE_BUTTON)
+        self.button.set_tooltip_text(tooltip)
+
+    def on_toggled(self, button):
+        """
+            Mutes or unmutes the volume
+        """
+        if button.get_active():
+            self.restore_volume = settings.get_option('player/volume', 1)
+            volume = 0
+        else:
+            volume = self.restore_volume
+
+        self.update_volume_icon(volume)
+
+        if self.restore_volume > 0:
+            settings.set_option('player/volume', volume)
+
+    def on_setting_change(self, event, sender, option):
+        """
+            Saves the restore volume and
+            changes the volume indicator
+        """
+        if option == 'player/volume':
+            volume = settings.get_option(option, 1)
+
+            if volume > 0:
+                self.button.set_active(False)
+
+            self.update_volume_icon(volume)
+
 BITMAP_CACHE = dict()
-def get_text_icon(widget, text, width, height, bgcolor='#456eac',   
+def get_text_icon(widget, text, width, height, bgcolor='#456eac',
     bordercolor=None):
     """
         Gets a bitmap icon with the specified text, width, and height
@@ -450,7 +495,7 @@ def get_text_icon(widget, text, width, height, bgcolor='#456eac',
     gc = pixmap.new_gc(foreground=black, background=white)
 
     if not bordercolor: bordercolor = black
-    else: 
+    else:
         bordercolor = colormap.alloc_color(gtk.gdk.color_parse(bordercolor))
     gc.set_foreground(bordercolor)
 
@@ -503,7 +548,7 @@ class Menu(gtk.Menu):
         image = gtk.Image()
         image.set_from_pixbuf(pixbuf)
         item.add(image)
-        
+
         if callback: item.connect('activate', callback, data)
         gtk.Menu.append(self, item)
         item.show_all()
@@ -631,55 +676,120 @@ class Menu(gtk.Menu):
         else:
             gtk.Menu.popup(self, *e)
 
-class StatusBar(object):
+class Statusbar(object):
     """
-        A basic statusbar to replace gtk.StatusBar
+        Convenient access to multiple status labels
     """
-    def __init__(self, label):
+    def __init__(self, status_bar):
         """
-            Initializes the bar
-            
-            @param label: the gtk.Label to use for setting status messages
+            Initialises the status bar
         """
-        self.label = label
+        # The first child of the status bar is a frame containing a label. We
+        # create an HBox, pack it inside the frame, and move the label and other
+        # widgets of the status bar into it.
+        self.status_bar = status_bar
+        children = status_bar.get_children()
+        frame = children[0]
+        label = frame.child
+        hbox = gtk.HBox(False, 0)
+        frame.remove(label)
+        hbox.pack_start(label, True, True)
+        frame.add(hbox)
+        for widget in children[1:]:
+            # Bug in old PyGTK versions: Statusbar.remove hides
+            # Container.remove.
+            gtk.Container.remove(status_bar, widget)
+            hbox.pack_start(widget, False, True)
 
-    def set_label(self, message, timeout=0):
-        """
-            Sets teh status label
-        """
-        self.label.set_label(message)
-        if timeout:
-            gobject.timeout_add(timeout, self.clear)
+        self.track_count_label = children[1]
+        self.queue_count_label = children[2].child
 
-    def clear(self, *e):
-        """
-            Clears the label
-        """
-        self.label.set_label('')
+        self.context_id = self.status_bar.get_context_id('status')
+        self.message_ids = []
 
+    def set_status(self, status, timeout=0):
+        """
+            Sets the status message
+        """
+        self.message_ids += [self.status_bar.push(self.context_id, status)]
+
+        if timeout > 0:
+            gobject.timeout_add_seconds(timeout, self.clear_status)
+
+    def clear_status(self):
+        """
+            Clears the status message
+        """
+        try:
+            for message_id in self.message_ids:
+                self.status_bar.remove_message(self.context_id, message_id)
+        except AttributeError:
+            for message_id in self.message_ids:
+                self.status_bar.remove(self.context_id, message_id)
+
+        del self.message_ids[:]
+        self.message_ids = []
+
+    def set_track_count(self, playlist_count=0, collection_count=0):
+        """
+            Sets the track count
+        """
+        status = _("%(playlist_count)d showing, "
+            "%(collection_count)d in collection") % {
+            'playlist_count': playlist_count,
+            'collection_count': collection_count
+        }
+
+        self.track_count_label.set_label(status)
+
+    def set_queue_count(self, queue_count=0):
+        """
+            Sets the queue count
+        """
+        if queue_count > 0:
+            self.queue_count_label.set_text(_("(%d queued)") % queue_count)
+            self.queue_count_label.set_no_show_all(False)
+            self.queue_count_label.show_all()
+        else:
+            self.queue_count_label.hide_all()
+            self.queue_count_label.set_no_show_all(True)
 
 class MenuRatingWidget(gtk.MenuItem):
     """
         A rating widget that can be used as a menu entry
+
+        @param: _get_tracks is a function that should return a list of tracks
+           linked to this widget.
+
+        @param: _get_tracks_rating is a function that should return an int
+           representing the rating of the tracks which are meant to be linked to
+           this widget. Should return 0 if two tracks have a different rating or
+           if it contains over miscellaneous/rating_widget_tracks_limit tracks.
+           Default limit: 100
     """
 
-    def __init__(self, _get_tracks):
+    def __init__(self, _get_tracks, _get_tracks_rating):
         gtk.MenuItem.__init__(self)
-        
+
         self._get_tracks = _get_tracks
-        
+        self._get_tracks_rating = _get_tracks_rating
+        self._last_calculated_rating = self._get_tracks_rating()
+
         self.hbox = gtk.HBox(spacing=3)
         self.hbox.pack_start(gtk.Label(_("Rating:")), False, False)
         self.image = gtk.image_new_from_pixbuf(
-            self._get_rating_pixbuf(0))
+            self._get_rating_pixbuf(self._last_calculated_rating))
         self.hbox.pack_start(self.image, False, False, 12)
-        
+
         self.add(self.hbox)
-        
+
         self.connect('button-release-event', self._update_rating)
         self.connect('motion-notify-event', self._motion_notify)
         self.connect('leave-notify-event', self._leave_notify)
 
+        # This may be useful when rating is changed from outside of the GUI
+        event.add_callback(self.on_rating_change, 'rating_changed')
+
     def _motion_notify(self, widget, e):
         steps = settings.get_option('miscellaneous/rating_steps', 5)
         (x, y) = e.get_coords()
@@ -704,19 +814,19 @@ class MenuRatingWidget(gtk.MenuItem):
             Resets the rating if the widget
             is left without clicking
         """
-        tracks = self._get_tracks()
-        if tracks and tracks[0]:
-            self.image.set_from_pixbuf(
-                self._get_rating_pixbuf(tracks[0].get_rating()))
-            self.queue_draw()
+        self.image.set_from_pixbuf(
+            self._get_rating_pixbuf(self._last_calculated_rating))
+        self.queue_draw()
 
     def _update_rating(self, widget, e):
         """
             Updates the rating of the tracks for this
             widget, meant to be used with a click event
         """
-        tracks = self._get_tracks()
-        if tracks and tracks[0]:
+        event.remove_callback(self.on_rating_change, 'rating_changed')
+
+        trs = self._get_tracks()
+        if trs and trs[0]:
             steps = settings.get_option('miscellaneous/rating_steps', 5)
             (x, y) = e.get_coords()
             (u, v) =  self.translate_coordinates(self.image, int(x), int(y))
@@ -726,30 +836,39 @@ class MenuRatingWidget(gtk.MenuItem):
                 r = (u / 12) + 1
             else:
                 r = -1
-            
+
             if r >= 0:
-                if r == tracks[0].get_rating() and \
-                    self._all_ratings_equal(tracks):
+                if r == self._last_calculated_rating:
                     r = 0
-                
-                for track in tracks:
+
+                for track in trs:
                     track.set_rating(r)
-                
+
                 event.log_event('rating_changed', self, r)
+                self._last_calculated_rating = r
+
+        event.add_callback(self.on_rating_change, 'rating_changed')
 
     def _get_rating_pixbuf(self, num):
         """
             Returns the pixbuf for num
         """
         return rating.rating_images[num]
-            
+
+    def on_rating_change(self, type = None, object = None, data = None):
+        """
+            Handles possible changes of track ratings
+        """
+        self._last_calculated_rating = self._get_tracks_rating()
+        self.image.set_from_pixbuf(
+            self._get_rating_pixbuf(self._last_calculated_rating))
+        self.queue_draw ()
+
 def get_urls_for(items):
     """
         Returns the items' URLs
     """
-    return [urllib.quote(item.get_loc().encode(
-        common.get_default_encoding()), '/:')
-        for item in items]
+    return [item.get_loc_for_io() for item in items]
 
 def finish(repeat=True):
     """
@@ -759,20 +878,6 @@ def finish(repeat=True):
         gtk.main_iteration()
         if not repeat: break
 
-def on_slider_scroll(widget, ev):
-    """
-        Called when the user scrolls their mouse wheel over the volume bar
-    """
-    incr = widget.get_adjustment().page_size
-    # Modify default HScale up/down behaviour.
-    if ev.direction == gtk.gdk.SCROLL_DOWN:
-        widget.set_value(widget.get_value() - incr)
-        return True
-    elif ev.direction == gtk.gdk.SCROLL_UP:
-        widget.set_value(widget.get_value() + incr)
-        return True
-    return False
-
 def on_slider_key_press(widget, ev):
     """
         Called when the user presses a key when the volume bar is focused
