@@ -1,6 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen 
-#
-# Copyright (C) 2008-2009 Adam Olsen 
+# Copyright (C) 2008-2009 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,26 +15,18 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 #
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
-# from your version.
-#
-#
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
+# The developers of the Exaile media player hereby grant permission
+# for non-GPL compatible GStreamer and Exaile plugins to be used and
+# distributed together with GStreamer and Exaile. This permission is
+# above and beyond the permissions granted by the GPL license by which
+# Exaile is covered. If you modify this code, you may extend this
+# exception to your version of the code, but you are not obligated to
+# do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import gtk, gobject
+import gobject
+import gtk
+
 from xlgui import guiutil, commondialogs, rating
 from xl import event, playlist, xdg, settings
 from xl.nls import gettext as _
@@ -54,7 +44,7 @@ class GenericTrackMenu(guiutil.Menu):
     }
     def __init__(self):
         guiutil.Menu.__init__(self)
-        
+
         self._add_queue_pixbuf()
         self._create_menu()
 
@@ -159,21 +149,21 @@ class PlaylistMenu(GenericTrackMenu):
     def __init__(self, playlist, playlists):
         GenericTrackMenu.__init__(self)
         self.playlist = playlist
-        
-        self.rating_item = guiutil.MenuRatingWidget( 
-            self.playlist.get_selected_tracks)
-        
+
+        self.rating_item = guiutil.MenuRatingWidget(
+            self.playlist.get_selected_tracks, self.playlist.get_tracks_rating)
+
         self.append_item(self.rating_item)
 
         self.add_playlist_menu = AddToPlaylistMenu(playlist, playlists)
-        self.append_menu(_('Add to custom playlist'), 
+        self.append_menu(_('Add to custom playlist'),
             self.add_playlist_menu, 'gtk-add')
         self.append_separator()
-        self.append(_('Remove'), lambda *e: self.remove_selected_tracks(), 
+        self.append(_('Remove'), lambda *e: self.remove_selected_tracks(),
             'gtk-remove')
         self.append_separator()
         self.append(_('Properties'), lambda *e: self.emit('properties'),
-            'gtk-properties') 
+            'gtk-properties')
 
         self.playlist_tab_menu = None
         # Defer menu setup until exaile is loaded
@@ -203,12 +193,13 @@ class PlaylistMenu(GenericTrackMenu):
         """
         from xlgui import main
         if self.playlist_tab_menu:
-          pagecount = main.get_playlist_nb().get_n_pages()
+          pagecount = main.get_playlist_notebook().get_n_pages()
           if settings.get_option('gui/show_tabbar', True) or pagecount > 1:
               self.playlist_tab_menu.hide_all()
           else:
               if pagecount == 1:
                   self.playlist_tab_menu.show_all()
+        self.rating_item.on_rating_change()
         GenericTrackMenu.popup(self, event)
 
 class PlaylistTabMenu(guiutil.Menu):
@@ -239,6 +230,7 @@ class TrackSelectMenu(GenericTrackMenu):
     """
     __gsignals__ = {
         'append-items': (gobject.SIGNAL_RUN_LAST, None, ()),
+        'properties': (gobject.SIGNAL_RUN_LAST, None, ()),
     }
     def __init__(self):
         """
@@ -254,6 +246,9 @@ class TrackSelectMenu(GenericTrackMenu):
             self.on_append_items(), 'gtk-add')
         self.queue_item = self.append(_('Queue Items'), lambda *e: self.on_queue(),
             'exaile-queue-icon')
+        self.append_separator()
+        self.append(_('Properties'), lambda *e: self.emit('properties'),
+            'gtk-properties')
 
     def on_append_items(self, selected=None):
         """
@@ -272,13 +267,14 @@ class RatedTrackSelectMenu(TrackSelectMenu):
         Menu for any panel that operates on selecting tracks
         including an option to rate tracks
     """
-    def __init__(self, get_selected_tracks):
-        
+    def __init__(self, tree_selection, get_selected_tracks, get_tracks_rating):
+
+        self.tree_selection = tree_selection
         self.rating_item = guiutil.MenuRatingWidget(
-            get_selected_tracks)
+            get_selected_tracks, get_tracks_rating)
 
         TrackSelectMenu.__init__(self)
-        
+
     def _create_menu(self):
         """
             Actually adds the menu items
@@ -286,6 +282,23 @@ class RatedTrackSelectMenu(TrackSelectMenu):
         TrackSelectMenu._create_menu(self)
         gtk.Menu.append(self, self.rating_item)
         self.rating_item.show_all()
+        self.changed_id = -1
+
+
+    def popup(self, event):
+        """
+            Displays the menu
+        """
+        self.rating_item.on_rating_change()
+        self.changed_id = self.tree_selection.connect('changed', self.rating_item.on_rating_change)
+        TrackSelectMenu.popup(self, event)
+
+    def popdown(self, event):
+        """
+            Displays the menu
+        """
+        self.tree_selection.disconnect(self.changed_id)
+        TrackSelectMenu.popdown(self, event)
 
 class CollectionPanelMenu(RatedTrackSelectMenu):
     __gsignals__ = {
@@ -296,12 +309,12 @@ class CollectionPanelMenu(RatedTrackSelectMenu):
 
     def _create_menu(self):
         RatedTrackSelectMenu._create_menu(self)
-        self.delete_item = self.append(_('Delete track'), 
+        self.delete_item = self.append(_('Delete track'),
                 lambda *e: self.on_delete_track(), 'gtk-delete')
 
     def on_delete_track(self):
         self.emit('delete-items')
-    
+
 
 
 # these are stubbs for now
