@@ -1,6 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen 
-#
-# Copyright (C) 2008-2009 Adam Olsen 
+# Copyright (C) 2008-2009 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,33 +15,24 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 #
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
-# from your version.
-#
-#
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
+# The developers of the Exaile media player hereby grant permission
+# for non-GPL compatible GStreamer and Exaile plugins to be used and
+# distributed together with GStreamer and Exaile. This permission is
+# above and beyond the permissions granted by the GPL license by which
+# Exaile is covered. If you modify this code, you may extend this
+# exception to your version of the code, but you are not obligated to
+# do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import gtk, gobject
+import gobject
+import gtk
+import threading
+
 from xlgui import panel, guiutil, commondialogs, menu
 from xlgui import playlist as guiplaylist
 import xlgui.panel.playlists as playlistpanel
-from xl import xdg, event, common
-from xl import settings
+from xl import xdg, event, common, settings
 import xl.radio
-import threading
 import xl.playlist
 from xl.nls import gettext as _
 
@@ -59,17 +48,17 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         'append-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
         'queue-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
     }
-    gladeinfo = ('radio_panel.glade', 'RadioPanelWindow')
+    ui_info = ('radio_panel.ui', 'RadioPanelWindow')
     _radiopanel = None
 
-    def __init__(self, parent, collection, 
+    def __init__(self, parent, collection,
         radio_manager, station_manager):
         """
             Initializes the radio panel
         """
         panel.Panel.__init__(self, parent)
         playlistpanel.BasePlaylistPanelMixin.__init__(self)
-      
+
         self.collection = collection
         self.manager = radio_manager
         self.playlist_manager = station_manager
@@ -98,7 +87,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         for name in self.playlist_manager.playlists:
             pl = self.playlist_manager.get_playlist(name)
             if pl is not None:
-                self.playlist_nodes[pl] = self.model.append(self.custom, 
+                self.playlist_nodes[pl] = self.model.append(self.custom,
                     [self.playlist_image,
                     pl.name, pl])
                 self._load_playlist_nodes(pl)
@@ -106,7 +95,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
 
         for name, value in self.manager.stations.iteritems():
             self.add_driver(value)
-    
+
     def _add_driver_cb(self, type, object, driver):
         self.add_driver(driver)
 
@@ -116,11 +105,11 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         """
         node = self.model.append(self.radio_root, [self.folder, str(driver), driver])
         self.nodes[driver] = node
-        self.load_nodes[driver] = self.model.append(node, [self.refresh_image, 
+        self.load_nodes[driver] = self.model.append(node, [self.refresh_image,
             _('Loading streams...'), None])
         self.tree.expand_row(self.model.get_path(self.radio_root), False)
 
-        if settings.get_option('gui/radio/%s_station_expanded' % 
+        if settings.get_option('gui/radio/%s_station_expanded' %
                 driver.name, False):
             self.tree.expand_row(self.model.get_path(node), False)
 
@@ -139,21 +128,21 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         """
             Sets up the various widgets required for this panel
         """
-        self.status = self.xml.get_widget('status_label')
+        self.status = self.builder.get_object('status_label')
 
     @guiutil.idle_add()
     def _set_status(self, message, timeout=0):
         self.status.set_text(message)
 
         if timeout:
-            gobject.timeout_add(timeout, self._set_status, _('Idle.'), 0)
+            gobject.timeout_add_seconds(timeout, self._set_status, '', 0)
 
     def _connect_events(self):
         """
             Connects events used in this panel
         """
         self.track_menu.connect('remove-track', lambda *e:
-            self.remove_selected_tracks())
+            self.remove_selected_track())
 
         menu = self.playlist_menu
         menu.connect('add-playlist', self._on_add_button_clicked)
@@ -162,7 +151,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
             self.emit('append-items', self.get_selected_tracks()))
         menu.connect('queue-items', lambda *e:
             self.emit('queue-items', self.get_selected_tracks()))
-        menu.connect('open-playlist', lambda *e: 
+        menu.connect('open-playlist', lambda *e:
             self.open_selected_playlist())
         menu.connect('export-playlist', lambda widget, path:
             self.export_selected_playlist(path))
@@ -172,7 +161,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
             self.remove_selected_playlist())
 
 
-        self.xml.signal_autoconnect({
+        self.builder.connect_signals({
             'on_add_button_clicked': self._on_add_button_clicked,
         })
         self.tree.connect('row-expanded', self.on_row_expand)
@@ -180,9 +169,9 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         self.tree.connect('row-activated', self.on_row_activated)
         self.tree.connect('key-release-event', self.on_key_released)
 
-        event.add_callback(self._add_driver_cb, 'station_added', 
+        event.add_callback(self._add_driver_cb, 'station_added',
                 self.manager)
-        event.add_callback(self._remove_driver_cb, 'station_removed', 
+        event.add_callback(self._remove_driver_cb, 'station_removed',
                 self.manager)
 
     def _on_add_button_clicked(self, *e):
@@ -198,16 +187,16 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         if result == gtk.RESPONSE_OK:
             (name, uri) = dialog.get_values()
             self._do_add_playlist(name, uri)
-    
+
     @common.threaded
     def _do_add_playlist(self, name, uri):
-        from xl import playlist, track
+        from xl import playlist, trax
         if playlist.is_valid_playlist(uri):
             pl = playlist.import_playlist(uri)
             pl.name = name
         else:
             pl = playlist.Playlist(name)
-            tracks = track.get_tracks_from_uri(uri)
+            tracks = trax.get_tracks_from_uri(uri)
             try:
                 pl.add_tracks(tracks)
             # Catch empty directories
@@ -219,7 +208,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
 
     @guiutil.idle_add()
     def _add_to_tree(self, pl):
-        self.playlist_nodes[pl] = self.model.append(self.custom, 
+        self.playlist_nodes[pl] = self.model.append(self.custom,
             [self.playlist_image, pl.name, pl])
         self._load_playlist_nodes(pl)
 
@@ -227,7 +216,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         """
             Sets up the tree that displays the radio panel
         """
-        box = self.xml.get_widget('RadioPanel')
+        box = self.builder.get_object('RadioPanel')
         self.tree = guiutil.DragTreeView(self, True, True)
         self.tree.set_headers_visible(False)
 
@@ -250,14 +239,14 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         self.model = gtk.TreeStore(gtk.gdk.Pixbuf, str, object)
         self.tree.set_model(self.model)
 
-        self.open_folder = guiutil.get_icon('gnome-fs-directory-accept')
         self.track = gtk.gdk.pixbuf_new_from_file(
             xdg.get_data_path('images/track.png'))
-        self.folder = guiutil.get_icon('gnome-fs-directory')
+        self.folder = self.tree.render_icon(
+            gtk.STOCK_DIRECTORY, gtk.ICON_SIZE_SMALL_TOOLBAR)
         self.refresh_image = guiutil.get_icon('gtk-refresh')
 
-        self.custom = self.model.append(None, [self.open_folder, _("Saved Stations"), None])
-        self.radio_root = self.model.append(None, [self.open_folder, _("Radio "
+        self.custom = self.model.append(None, [self.folder, _("Saved Stations"), None])
+        self.radio_root = self.model.append(None, [self.folder, _("Radio "
             "Streams"), None])
 
         scroll = gtk.ScrolledWindow()
@@ -288,7 +277,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         menu = guiutil.Menu()
         menu.append(_("Refresh"), self.on_reload, 'gtk-refresh')
         return menu
-        
+
     def on_key_released(self, widget, event):
         """
             Called when a key is released in the tree
@@ -313,19 +302,19 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
                 elif isinstance(item, playlistpanel.TrackWrapper):
                     gtk.Menu.popup(self.track_menu, None, None, None, 0, event.time)
             return True
-        
+
         if event.keyval == gtk.keysyms.Left:
             (mods,paths) = self.tree.get_selection().get_selected_rows()
             if paths and paths[0]:
                 self.tree.collapse_row(paths[0])
             return True
-        
+
         if event.keyval == gtk.keysyms.Right:
             (mods,paths) = self.tree.get_selection().get_selected_rows()
             if paths and paths[0]:
                 self.tree.expand_row(paths[0], False)
             return True
-        
+
         return False
 
     def button_press(self, widget, event):
@@ -369,12 +358,12 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         #TODO this might change if we are allowed to change the order of radio
         if tv == context.get_source_widget():
             context.drop_finish(False, etime)
-            return  
-        
+            return
+
         locs = list(selection.get_uris())
-        
+
         path = self.tree.get_path_at_pos(x, y)
-        if path: 
+        if path:
             # Add whatever we received to the playlist at path
             iter = self.model.get_iter(path[0])
             current_playlist = self.model.get_value(iter, 2)
@@ -401,18 +390,18 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         """
         # If the user dragged files prompt for a new playlist name
         # else if they dragged a playlist add the playlist
-        
+
         #We don't want the tracks in the playlists to be added to the
         # master tracks list so we pass in False
         (tracks, playlists) = self.tree.get_drag_data(locs, False)
         #First see if they dragged any playlist files
         for new_playlist in playlists:
             self.model.append(self.custom, [self.playlist_image,
-                new_playlist.name, new_playlist]) 
+                new_playlist.name, new_playlist])
             # We are adding a completely new playlist with tracks so we save it
             self.playlist_manager.save_playlist(new_playlist, overwrite=True)
-                
-        #After processing playlist proceed to ask the user for the 
+
+        #After processing playlist proceed to ask the user for the
         #name of the new playlist to add and add the tracks to it
         if len(tracks) > 0:
             dialog = commondialogs.TextEntryDialog(
@@ -425,12 +414,12 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
                     #Create the playlist from all of the tracks
                     new_playlist = xl.playlist.Playlist(name)
                     new_playlist.add_tracks(tracks)
-                    self.playlist_nodes[new_playlist] = self.model.append(self.custom, 
+                    self.playlist_nodes[new_playlist] = self.model.append(self.custom,
                         [self.playlist_image,
                         new_playlist.name, new_playlist])
                     self.tree.expand_row(self.model.get_path(self.custom), False)
                     # We are adding a completely new playlist with tracks so we save it
-                    self.playlist_manager.save_playlist(new_playlist)                
+                    self.playlist_manager.save_playlist(new_playlist)
                     self._load_playlist_nodes(new_playlist)
 
     def drag_get_data(self, tv, context, selection_data, info, time):
@@ -442,15 +431,15 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
             tracks = pl.get_tracks()
         else:
             tracks = self.get_selected_tracks()
-           
+
         if not tracks: return
 
         for track in tracks:
-            guiutil.DragTreeView.dragged_data[track.get_loc()] = track
-        
+            guiutil.DragTreeView.dragged_data[track.get_loc_for_io()] = track
+
         urls = guiutil.get_urls_for(tracks)
         selection_data.set_uris(urls)
-    
+
     def drag_data_delete(self, *e):
         """
             stub
@@ -467,7 +456,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         (model, paths) = info
         iter = self.model.get_iter(paths[0])
         object = self.model.get_value(iter, 2)
-        
+
         try:
             self.loaded_nodes.remove(self.nodes[object])
         except ValueError: pass
@@ -494,12 +483,12 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         driver = self.model.get_value(iter, 2)
 
         if not isinstance(driver, xl.playlist.Playlist):
-            self.model.set_value(iter, 0, self.open_folder)
+            self.model.set_value(iter, 0, self.folder)
 
         if isinstance(driver, xl.radio.RadioStation) or \
             isinstance(driver, xl.radio.RadioList):
             if not self.nodes[driver] in self.loaded_nodes:
-                self._load_station(iter, driver)        
+                self._load_station(iter, driver)
 
         if isinstance(driver, xl.radio.RadioStation):
             self.set_station_expanded_value(driver.name, True)
@@ -516,7 +505,7 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         if isinstance(driver, xl.radio.RadioStation):
             self.set_station_expanded_value(driver.name, False)
 
-    @common.threaded 
+    @common.threaded
     def _load_station(self, iter, driver):
         """
             Loads a radio station
@@ -531,12 +520,12 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
             try:
                 lists = driver.get_lists(no_cache=no_cache)
             except RadioException, e:
-                self._set_status(str(e), 2000)
+                self._set_status(str(e), 2)
         else:
             try:
                 lists = driver.get_items(no_cache=no_cache)
             except RadioException, e:
-                self._set_status(str(e), 2000)
+                self._set_status(str(e), 2)
 
         if not lists: return
         gobject.idle_add(self._done_loading, iter, driver, lists)
@@ -547,11 +536,11 @@ class RadioPanel(panel.Panel, playlistpanel.BasePlaylistPanelMixin):
         """
         self.loaded_nodes.append(self.nodes[object])
         for item in items:
-            if isinstance(item, xl.radio.RadioList): 
+            if isinstance(item, xl.radio.RadioList):
                 node = self.model.append(self.nodes[object], [self.folder, item.name, item])
                 self.nodes[item] = node
-                self.load_nodes[item] = self.model.append(node, [self.refresh_image, 
-                    _("Loading streams..."), None]) 
+                self.load_nodes[item] = self.model.append(node, [self.refresh_image,
+                    _("Loading streams..."), None])
             else:
                 self.model.append(self.nodes[object], [self.track, item.name,
                     item])
