@@ -21,12 +21,12 @@ import os
 
 name = _('IPython Console')
 basedir = os.path.dirname(os.path.realpath(__file__))
-glade = os.path.join(basedir, 'ipconsole_prefs.glade')
+ui = os.path.join(basedir, 'ipconsole_prefs.ui')
 
 class OpacityPreference(widgets.ScalePrefsItem):
     default = 80.0
     name = 'plugin/ipconsole/opacity'
 
 class FontPreference(widgets.FontButtonPrefsItem):
-    default = 'Luxi Mono 10'
+    default = 'Monospace 10'
     name = 'plugin/ipconsole/font'
