@@ -18,12 +18,12 @@
 import copy, gobject, gtk, pango
 from string import Template
 from xl import event, settings
-from xl.track import Track
 from xl.nls import gettext as _
+from xl.trax import Track
+from xlgui.guiutil import get_workarea_size
 from xlgui.playlist import Playlist
-from minimodeprefs import get_workarea_size
 
-class MMMenuItem(gtk.ImageMenuItem):
+class MenuItem(gtk.ImageMenuItem):
     """
         Convenience wrapper, allows switching to mini mode
     """
@@ -31,9 +31,18 @@ class MMMenuItem(gtk.ImageMenuItem):
         gtk.ImageMenuItem.__init__(self, stock_id='exaile-minimode')
         self.child.set_label(_('Mini Mode'))
 
-        self._callback_id = self.connect('activate', callback)
+        self._activate_id = self.connect('activate', callback)
         event.add_callback(self.on_stock_icon_added, 'stock_icon_added')
 
+    def destroy(self):
+        """
+            Various cleanups
+        """
+        self.disconnect(self._activate_id)
+        event.remove_callback(self.on_stock_icon_added, 'stock_icon_added')
+
+        gtk.ImageMenuItem.destroy(self)
+
     def on_stock_icon_added(self, iconmanager, stock_id):
         """
             Handles deferred icon load
@@ -42,120 +51,149 @@ class MMMenuItem(gtk.ImageMenuItem):
             self.set_image(gtk.gdk.image_new_from_stock(stock_id))
             event.remove_callback(self.on_stock_icon_added, 'stock_icon_added')
 
-    def destroy(self):
-        """
-            Does cleanup
-        """
-        self.disconnect(self._callback_id)
-        self.image.destroy()
-        gtk.ImageMenuItem.destroy(self)
+class KeyExistsError(KeyError):
+    def __init__(self, error):
+        KeyError.__init__(self, error)
 
-class MMBox(gtk.HBox):
+class WidgetBox(gtk.HBox):
     """
-        Convenience wrapper around gtk.HBox, allows
-        for simple access of contained widgets
+        Wrapper class, allows for identification
+        of and simple access to contained widgets
+        Keeps track of registered types
     """
     def __init__(self, homogeneous=False, spacing=0):
         gtk.HBox.__init__(self, homogeneous, spacing)
 
-        self._removed_widgets = []
+        self.__register = {}
+        self.__widgets = {}
 
-    def pack_start(self, child):
+    def destroy(self):
         """
-            Inserts a child into this box
+            Various cleanups
         """
-        if not isinstance(child, MMWidget):
-            raise TypeError(
-                '%s is not instance of %s' % (child, MMWidget))
-        gtk.HBox.pack_start(self, child, expand=False, fill=False)
+        for id, widget in self.__widgets.iteritems():
+            widget.destroy()
+
+        gtk.HBox.destroy(self)
+
+    def register_widget(self, id, type, arguments=[]):
+        """
+            Registers a widget
+        """
+        if id in self.__register:
+            raise KeyExistsError, id
+
+        self.__register[id] = (type, arguments)
+
+    def register_widgets(self, widgets):
+        """
+            Registers multiple widgets at once
+        """
+        for id, (type, arguments) in widgets.iteritems():
+            self.register_widget(id, type, arguments)
+
+    def add_widget(self, id):
+        """
+            Adds a widget to the box, moves
+            it to the end if already present
+        """
+        if id not in self.__register:
+            raise KeyError, id
+
+        if id in self.__widgets:
+            widget = self.__widgets[id]
+            self.reorder_child(widget, -1)
+        else:
+            type, arguments = self.__register[id]
+            widget = type(*arguments)
+            self.__widgets[id] = widget
+            self.pack_start(widget, expand=False, fill=False)
+
+    def remove_widget(self, id):
+        """
+            Removes a widget from the box
+        """
+        if id not in self.__register:
+            raise KeyError, id
+
+        if id not in self.__widgets:
+            raise KeyError, id
+
+        self.remove(self.__widgets[id])
+        self.__widgets[id].destroy()
+        del self.__widgets[id]
+
+    def update_widgets(self, ids):
+        """
+            Updates display and order of widgets
+        """
+        for id in self.__register.iterkeys():
+            if id not in ids:
+                try:
+                    self.remove_widget(id)
+                except KeyError:
+                    pass
+
+        for id in ids:
+            try:
+                self.add_widget(id)
+            except KeyError:
+                pass
 
     def __getitem__(self, id):
         """
-            Returns a contained child
+            Returns a widget, allows for box['id']
         """
-        all_widgets = []
-        all_widgets += [widget for widget in self]
-        all_widgets += self._removed_widgets
-        for widget in all_widgets:
-            if widget.id == id:
-                return widget
-        raise KeyError
+        return self.__widgets[id]
 
-    def show_child(self, id):
-        """
-            Shows a contained child
-        """
-        for widget in self._removed_widgets:
-            if widget.id == id:
-                self.pack_start(widget)
-                self._removed_widgets.remove(widget)
-                return
-        raise KeyError('No widget with id %s' % id)
-
-    def show_all_children(self):
-        """
-            Shows all contained children
-        """
-        for widget in self.get_children():
-            self.show_child(widget.id)
-
-    def hide_child(self, id):
-        """
-            Hides a contained child
-        """
-        for widget in self.get_children():
-            if widget.id == id:
-                self._removed_widgets.append(widget)
-                self.remove(widget)
-                return
-        raise KeyError('No widget with id %s' % id)
-
-    def hide_all_children(self):
-        """
-            Hides all contained children
-        """
-        for widget in self.get_children():
-            self.hide_child(widget.id)
-
-class MMWidget(gtk.Widget):
-    """
-        Wrapper for gtk.Widget,
-        allows for identification of widgets
-    """
-    def __init__(self, id):
-        gtk.Widget.__init__(self)
-        self.id = id
-
-class MMButton(MMWidget, gtk.Button):
+class Button(gtk.Button):
     """
         Convenience wrapper around gtk.Button
     """
-    def __init__(self, id, stock_id, tooltip_text, callback):
-        MMWidget.__init__(self, id)
+    def __init__(self, stock_id, tooltip_text, callback):
         gtk.Button.__init__(self)
 
         self.image = gtk.image_new_from_stock(stock_id, gtk.ICON_SIZE_BUTTON)
         self.set_image(self.image)
         self.set_tooltip_text(tooltip_text)
         self.set_relief(gtk.RELIEF_NONE)
+        self.set_focus_on_click(False)
 
-        self.connect('clicked', callback)
+        self._clicked_id = self.connect('clicked', callback)
 
-class MMPlayPauseButton(MMButton):
+    def destroy(self):
+        """
+            Various cleanups
+        """
+        self.disconnect(self._clicked_id)
+
+        gtk.Button.destroy(self)
+
+class PlayPauseButton(Button):
     """
-        Special MMButton which automatically sets its
+        Special Button which automatically sets its
         appearance depending on the current playback state
     """
     def __init__(self, player, callback):
-        MMButton.__init__(self, 'play_pause', 'gtk-media-play',
+        Button.__init__(self, 'gtk-media-play',
             _('Start Playback'), callback)
 
         self.player = player
         self.update_state()
 
-        event.add_callback(self.on_playback_state_change, 'playback_player_start')
-        event.add_callback(self.on_playback_state_change, 'playback_toggle_pause')
         event.add_callback(self.on_playback_state_change, 'playback_player_end')
+        event.add_callback(self.on_playback_state_change, 'playback_track_start')
+        event.add_callback(self.on_playback_state_change, 'playback_toggle_pause')
+
+    def destroy(self):
+        """
+            Various cleanups
+        """
+        event.remove_callback(self.on_playback_state_change, 'playback_player_end')
+        event.remove_callback(self.on_playback_state_change, 'playback_track_start')
+        event.remove_callback(self.on_playback_state_change, 'playback_toggle_pause')
+
+        Button.destroy(self)
 
     def update_state(self):
         """
@@ -180,12 +218,11 @@ class MMPlayPauseButton(MMButton):
         """
         self.update_state()
 
-class MMVolumeButton(MMWidget, gtk.VolumeButton):
+class VolumeButton(gtk.VolumeButton):
     """
         Wrapper class around gtk.VolumeButton
     """
     def __init__(self, player, callback):
-        MMWidget.__init__(self, 'volume')
         gtk.VolumeButton.__init__(self)
 
         self.player = player
@@ -194,9 +231,17 @@ class MMVolumeButton(MMWidget, gtk.VolumeButton):
         self.set_adjustment(adjustment)
 
         self._changed_callback = callback
+        self._value_changed_id = self.connect('value-changed', self.on_change)
+        self._expose_event_id = self.connect('expose-event', self.on_expose)
 
-        self.connect('value-changed', self.on_change)
-        self.connect('expose-event', self.on_expose)
+    def destroy(self):
+        """
+            Various cleanups
+        """
+        self.disconnect(self._value_changed_id)
+        self.disconnect(self._expose_event_id)
+
+        gtk.VolumeButton.destroy(self)
 
     def on_change(self, *e):
         """
@@ -213,17 +258,89 @@ class MMVolumeButton(MMWidget, gtk.VolumeButton):
         self.set_value(self.player.get_volume() / 100.0)
         self._updating = False
 
-class MMPlaylistButton(MMWidget, gtk.ToggleButton):
+class AttachedWindow(gtk.Window):
+    """
+        A window attachable to arbitrary widgets,
+        follows the movement of its parent
+    """
+    def __init__(self, parent):
+        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
+
+        self.set_decorated(False)
+        self.set_property('skip-taskbar-hint', True)
+        self.set_size_request(350, 400)
+
+        self.parent_widget = parent
+
+        self._configure_id = None
+        self._parent_realize_id = self.parent_widget.connect(
+            'realize', self.on_parent_realize)
+
+    def destroy(self):
+        """
+            Various cleanups
+        """
+        if self._configure_id is not None:
+            self.disconnect(self._configure_id)
+
+        self.disconnect(self._parent_realize_id)
+
+        gtk.Window.destroy(self)
+
+    def update_location(self):
+        """
+            Makes sure the window is
+            always fully visible
+        """
+        workarea_width, workarea_height = get_workarea_size() # 1280, 974
+        width, height = self.size_request() #  350, 400
+        # FIXME: AttributeError: 'NoneType' object has no attribute 'get_origin'
+        parent_window_x, parent_window_y = self.parent_widget.get_window().get_origin()
+        parent_x, parent_y, parent_width, parent_height = self.parent_widget.get_allocation()
+        parent_x, parent_y = parent_window_x + parent_x, parent_window_y + parent_y
+
+        # E.g.       1280 - 1000    < 350
+        if workarea_width - parent_x < width:
+            #           1000 + 150          - 350 = 800
+            x = parent_x + parent_width - width # Aligned right
+        else:
+            x = parent_x # Aligned left
+
+        # E.g.         974 - 800     < 400
+        if workarea_height - parent_y < height:
+            #            800 - 400 = 400
+            y = parent_y - height # Aligned top
+        else:
+            y = parent_y + parent_height # Aligned bottom
+
+        self.move(x, y)
+
+    def on_parent_realize(self, parent):
+        """
+            Prepares the window to
+            follow its parent window
+        """
+        if self._configure_id is None:
+            self._configure_id = parent.get_toplevel().connect(
+                'configure-event', self.on_parent_window_configure)
+
+    def on_parent_window_configure(self, *e):
+        """
+            Handles movement of the topmost window
+        """
+        self.update_location()
+
+class PlaylistButton(gtk.ToggleButton):
     """
         Displays the current track title and
         the current playlist on activation
         Also allows for drag and drop of files
         to add them to the playlist
     """
-    def __init__(self, main, queue, playlist, change_callback, format_callback=None):
-        MMWidget.__init__(self, 'playlist_button')
+    def __init__(self, main, queue, formatter, change_callback):
         gtk.ToggleButton.__init__(self, '')
 
+        self.set_focus_on_click(False)
         self.set_size_request(150, -1)
         box = gtk.HBox()
         self.arrow = gtk.Arrow(gtk.ARROW_RIGHT, gtk.SHADOW_OUT)
@@ -235,76 +352,67 @@ class MMPlaylistButton(MMWidget, gtk.ToggleButton):
         self.add(box)
 
         self.main = main
-        self.queue = queue
-        self.formatter = MMTrackFormatter()
-        self.popup = gtk.Window(gtk.WINDOW_TOPLEVEL)
-        self.popup.set_decorated(False)
-        self.popup.set_size_request(350, 400)
-        self.playlist = Playlist(main, queue, playlist)
+        self.formatter = formatter
+        playlist = self.main.get_selected_playlist()
+        self.playlist = Playlist(main, queue, playlist.playlist)
+        self.playlist.model = playlist.model
+        self.playlist.list.set_model(self.playlist.model)
+        self.playlist.scroll.set_property('shadow-type', gtk.SHADOW_IN)
+        self.popup = AttachedWindow(self)
         self.popup.add(self.playlist)
 
-        self._updating = False
-        self._parent_configure_id = None
+        self._dirty = False
+        self._drag_shown = False
         self._parent_hide_id = None
         self._drag_motion_id = None
 
-        try:
-            self.formatter.connect('format-request', format_callback)
-        except TypeError:
-            pass
-
         self.drag_dest_set(gtk.DEST_DEFAULT_ALL,
             self.playlist.list.targets,
             gtk.gdk.ACTION_COPY |
             gtk.gdk.ACTION_DEFAULT |
             gtk.gdk.ACTION_MOVE)
 
-        self.connect('track-changed', change_callback)
-        self.connect('scroll-event', self.on_scroll)
-        self.connect('toggled', self.on_toggled)
-        self.connect_object('drag-data-received',
-            self.playlist.drag_data_received, self.playlist.list)
-        self.connect('drag-leave', self.on_drag_leave)
-        self.connect('drag-motion', self.on_drag_motion)
-        event.add_callback(self.on_playlist_current_changed, 'playlist_current_changed')
-        event.add_callback(self.on_playback_start, 'playback_player_start')
-        event.add_callback(self.on_playback_end, 'playback_player_end')
+        self._track_changed_id = self.connect('track-changed',
+            change_callback)
+        self._scroll_event_id = self.connect('scroll-event',
+            self.on_scroll)
+        self._toggled_id = self.connect('toggled',
+            self.on_toggled)
+        self._drag_leave_id = self.connect('drag-leave',
+            self.on_drag_leave)
+        self._drag_motion_id = self.connect('drag-motion',
+            self.on_drag_motion)
+        self._drag_data_received_id = self.playlist.list.connect('drag-data-received',
+            self.on_playlist_drag_data_received)
+        self._switch_page_id = self.main.playlist_notebook.connect('switch-page',
+            self.on_playlist_notebook_switch)
+        self._format_changed_id = self.formatter.connect('format-changed',
+            self.on_format_changed)
+
         event.add_callback(self.on_track_start, 'playback_track_start')
-        event.add_callback(self.on_tracks_changed, 'tracks_added')
-        event.add_callback(self.on_tracks_changed, 'tracks_removed')
-        event.add_callback(self.on_tracks_changed, 'tracks_reordered')
+        event.add_callback(self.on_playback_start, 'playback_player_start')
+        event.add_callback(self.on_playback_start, 'playback_player_resume')
+        event.add_callback(self.on_playback_end, 'playback_player_end')
 
-    def move_popup(self):
+    def destroy(self):
         """
-            Makes sure the popup is always fully visible
+            Various cleanups
         """
-        workarea_width, workarea_height = get_workarea_size() # 1280, 974
-        popup_width, popup_height = self.popup.size_request() #  350, 400
-        buttonx, buttony, button_width, button_height = self.get_allocation()
-        parentx, parenty = self.get_window().get_origin()
-        buttonx, buttony = parentx + buttonx, parenty + buttony
+        self.disconnect(self._track_changed_id)
+        self.disconnect(self._scroll_event_id)
+        self.disconnect(self._toggled_id)
+        self.disconnect(self._drag_leave_id)
+        self.disconnect(self._drag_motion_id)
+        self.playlist.list.disconnect(self._drag_data_received_id)
+        self.main.playlist_notebook.disconnect(self._switch_page_id)
+        self.formatter.disconnect(self._format_changed_id)
 
-        # E.g.       1280 - 1000    < 350
-        if workarea_width - buttonx < popup_width:
-            #           1000 + 150          - 350 = 800
-            popupx = buttonx + button_width - popup_width # Aligned right
-        else:
-            popupx = buttonx # Aligned left
+        event.remove_callback(self.on_track_start, 'playback_track_start')
+        event.remove_callback(self.on_playback_start, 'playback_player_start')
+        event.remove_callback(self.on_playback_start, 'playback_player_resume')
+        event.remove_callback(self.on_playback_end, 'playback_player_end')
 
-        # E.g.         974 - 800     < 400
-        if workarea_height - buttony < popup_height:
-            #            800 - 400 = 400 
-            popupy = buttony - popup_height # Aligned top
-        else:
-            popupy = buttony + button_height # Aligned bottom
-
-        self.popup.move(popupx, popupy)
-
-    def set_tracks(self, tracks):
-        """
-            Replaces the tracks of the playlist
-        """
-        self.playlist._set_tracks(tracks)
+        gtk.ToggleButton.destroy(self)
 
     def set_label(self, text):
         """
@@ -319,71 +427,51 @@ class MMPlaylistButton(MMWidget, gtk.ToggleButton):
         """
         self.arrow.set(direction, gtk.SHADOW_OUT)
 
-    def on_playlist_current_changed(self, event, playlist, track):
+    def on_format_changed(self, formatter, format):
         """
-            Updates the currently selected track
+            Updates the playlist button
+            on track title format changes
         """
-        if playlist != self.playlist.playlist:
-            try:
-                pos = self.playlist.playlist.index(track)
-            except ValueError:
-                return
-            self.playlist.playlist.set_current_pos(pos)
+        track = self.playlist.playlist.get_current()
 
-    def on_playback_start(self, event, player, track):
-        """
-            Updates appearance on playback start
-        """
-        self.set_label(self.formatter.format(track))
-
-    def on_playback_end(self, event, player, track):
-        """
-            Updates appearance on playback state change
-        """
-        self.set_label('')
+        if track:
+            self.set_label(self.formatter.format(track))
 
     def on_track_start(self, event, player, track):
         """
-            Updates the cursor position
+            Updates appearance and cursor position
         """
+        self.set_label(self.formatter.format(track))
+
         if track in self.playlist.playlist.ordered_tracks:
             path = (self.playlist.playlist.index(track),)
-        
+
             if settings.get_option('gui/ensure_visible', True):
                 self.playlist.list.scroll_to_cell(path)
 
             gobject.idle_add(self.playlist.list.set_cursor, path)
 
-    def on_tracks_changed(self, event, playlist, *args):
+    def on_playback_start(self, event, player, track):
         """
-            Updates the local playlist as well as the
-            currently selected playlist in the main window
+            Updates appearance on playback start/resume
         """
-        tracks = playlist.get_tracks()
+        self.set_label(self.formatter.format(track))
 
-        if playlist == self.playlist.playlist:
-            self.main.get_selected_playlist()._set_tracks(tracks)
-            self.main.get_selected_playlist().playlist._set_ordered_tracks(tracks)
-        else:
-            self.playlist._set_tracks(tracks)
-            self.playlist.playlist._set_ordered_tracks(tracks)
-
-    def on_parent_configure(self, *e):
+    def on_playback_end(self, event, player, track):
         """
-            Passes parent window movement to the popup
+            Clears label
         """
-        self.move_popup()
+        self.set_label('')
 
     def on_parent_hide(self, parent):
         """
             Makes sure to hide the popup
         """
         self.set_active(False)
-        self.popup.hide()
 
     def on_scroll(self, togglebutton, event):
         """
-            Handles scrolling on the button
+            Switches tracks on scrolling
         """
         track = None
 
@@ -401,16 +489,12 @@ class MMPlaylistButton(MMWidget, gtk.ToggleButton):
         if self.popup.get_transient_for() is None:
             self.popup.set_transient_for(self.get_toplevel())
 
-        if self._parent_configure_id is None:
-            self._parent_configure_id = self.get_toplevel().connect(
-                'configure-event', self.on_parent_configure)
-
         if self._parent_hide_id is None:
             self._parent_hide_id = self.get_toplevel().connect(
                 'hide', self.on_parent_hide)
 
         if self.get_active():
-            self.move_popup()
+            self.popup.update_location()
             self.set_arrow_direction(gtk.ARROW_DOWN)
             self.popup.show()
         else:
@@ -432,49 +516,97 @@ class MMPlaylistButton(MMWidget, gtk.ToggleButton):
         """
         if self._drag_motion_id is None:
             self._drag_motion_id = gobject.timeout_add(500,
-                lambda: self.set_active(True))
+                self.drag_motion_finish)
 
-gobject.type_register(MMPlaylistButton)
-gobject.signal_new('track-changed', MMPlaylistButton,
+    def drag_motion_finish(self):
+        """
+            Shows the playlist
+        """
+        self.set_active(True)
+        self._drag_shown = True
+
+    def on_playlist_drag_data_received(self, *e):
+        """
+            Hides the playlist if it has
+            been opened via drag events
+        """
+        if self._drag_shown:
+            self.set_active(False)
+            self._drag_shown = False
+
+    def on_playlist_notebook_switch(self, notebook, page, page_num):
+        """
+            Updates the internal playlist
+        """
+        playlist = notebook.get_nth_page(page_num)
+
+        if playlist is not None:
+            self.playlist.model = playlist.model
+            self.playlist.list.set_model(self.playlist.model)
+
+gobject.type_register(PlaylistButton)
+gobject.signal_new('track-changed', PlaylistButton,
     gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
     (gobject.TYPE_PYOBJECT, ))
 
-class MMTrackSelector(MMWidget, gtk.ComboBox):
+class TrackSelector(gtk.ComboBox):
     """
         Control which updates its content automatically
         on playlist actions, track display is configurable
     """
-    def __init__(self, queue, changed_callback, format_callback=None):
-        MMWidget.__init__(self, 'track_selector')
+    def __init__(self, main, queue, formatter, changed_callback):
         gtk.ComboBox.__init__(self)
 
+        self.main = main
         self.queue = queue
-        self.list = gtk.ListStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING)
-        self.formatter = MMTrackFormatter()
-        self._updating = False
-        #self._changed_callback = changed_callback
-
+        self.formatter = formatter
+        self.list = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
         self.set_model(self.list)
         self.set_size_request(150, 0)
 
         textrenderer = gtk.CellRendererText()
         self.pack_start(textrenderer, expand=True)
+        self.set_attributes(textrenderer, text=0)
         self.set_cell_data_func(textrenderer, self.text_data_func)
 
-        try:
-            self.formatter.connect('format-request', format_callback)
-        except TypeError:
-            pass
+        self._updating = False
+        self._dirty = False
 
         self.update_track_list(self.queue.current_playlist)
 
-        self.connect('changed', self.on_change)
-        self.connect('track-changed', changed_callback)
+        self._track_changed_id = self.connect('track-changed',
+            changed_callback)
+        self._expose_event_id = self.connect('expose-event',
+            self.on_expose)
+        self._changed_id = self.connect('changed',
+            self.on_change)
+        self._switch_page_id = self.main.playlist_notebook.connect('switch-page',
+            self.on_playlist_notebook_switch)
+        self._format_changed_id = self.formatter.connect('format-changed',
+            self.on_format_changed)
+
         event.add_callback(self.on_playlist_current_changed, 'playlist_current_changed')
         event.add_callback(self.on_tracks_added, 'tracks_added')
         event.add_callback(self.on_tracks_removed, 'tracks_removed')
         event.add_callback(self.on_tracks_reordered, 'tracks_reordered')
 
+    def destroy(self):
+        """
+            Various cleanups
+        """
+        self.disconnect(self._track_changed_id)
+        self.disconnect(self._expose_event_id)
+        self.disconnect(self._changed_id)
+        self.main.playlist_notebook.disconnect(self._switch_page_id)
+        self.formatter.disconnect(self._format_changed_id)
+
+        event.remove_callback(self.on_playlist_current_changed, 'playlist_current_changed')
+        event.remove_callback(self.on_tracks_added, 'tracks_added')
+        event.remove_callback(self.on_tracks_removed, 'tracks_removed')
+        event.remove_callback(self.on_tracks_reordered, 'tracks_reordered')
+
+        gtk.ComboBox.destroy(self)
+
     def update_track_list(self, playlist=None, tracks=None):
         """
             Populates the track list based
@@ -532,18 +664,11 @@ class MMTrackSelector(MMWidget, gtk.ComboBox):
             Updates track titles and highlights
             the current track if the popup is shown
         """
-        title = model.get_value(iter, 1)
-
-        if title is None:
-            return
-
-        cell.set_property('text', title)
-
         active_iter = self.get_active_iter()
 
         if active_iter is not None:
-            track = model.get_value(iter, 0)
-            active_track = model.get_value(active_iter, 0)
+            track = model.get_value(iter, 1)
+            active_track = model.get_value(active_iter, 1)
             weight = pango.WEIGHT_NORMAL
 
             if self.get_property('popup-shown'):
@@ -552,14 +677,28 @@ class MMTrackSelector(MMWidget, gtk.ComboBox):
 
             cell.set_property('weight', weight)
 
+    def on_expose(self, widget, event):
+        """
+            Performs deferred tasks
+        """
+        if self._dirty:
+            self.update_track_list()
+            self._dirty = False
+
     def on_change(self, *e):
         """
             Wrapper function to prevent race conditions
         """
         if not self._updating:
-            #self._changed_callback(*e)
             self.emit('track-changed', self.get_active_track())
 
+    def on_format_changed(self, formatter, format):
+        """
+            Updates the track list
+            on track title format changes
+        """
+        self.update_track_list()
+
     def on_playlist_current_changed(self, event, playlist, track):
         """
             Updates the currently selected track
@@ -570,32 +709,51 @@ class MMTrackSelector(MMWidget, gtk.ComboBox):
         """
             Triggers update of the track list on track addition
         """
-        self.update_track_list(playlist, tracks)
+        if self.get_toplevel().get_property('visible'):
+            self.update_track_list(playlist, tracks)
+        else:
+            self._dirty = True
 
     def on_tracks_removed(self, event, playlist, (start, end, removed)):
         """
             Triggers update of the track list on track removal
         """
-        self.update_track_list(playlist)
+        if self.get_toplevel().get_property('visible'):
+            self.update_track_list(playlist)
+        else:
+            self._dirty = True
 
     def on_tracks_reordered(self, event, playlist, tracks):
         """
             Triggers update of the track list on track reordering
         """
-        self.update_track_list(playlist, tracks)
+        if self.get_toplevel().get_property('visible'):
+            self.update_track_list(playlist, tracks)
+        else:
+            self._dirty = True
 
-gobject.type_register(MMTrackSelector)
-gobject.signal_new('track-changed', MMTrackSelector,
+    def on_playlist_notebook_switch(self, notebook, page, page_num):
+        """
+            Updates the internal playlist
+        """
+        if self.get_toplevel().get_property('visible'):
+            page = notebook.get_nth_page(page_num)
+            if page is not None:
+                self.update_track_list(page.playlist)
+        else:
+            self._dirty = True
+
+gobject.type_register(TrackSelector)
+gobject.signal_new('track-changed', TrackSelector,
     gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
     (gobject.TYPE_PYOBJECT, ))
 
-class MMProgressBar(MMWidget, gtk.Alignment):
+class ProgressBar(gtk.Alignment):
     """
         Wrapper class which updates itself
         based on the current track
     """
     def __init__(self, player, callback):
-        MMWidget.__init__(self, 'progress_bar')
         gtk.Alignment.__init__(self)
         self.set_padding(3, 3, 0, 0)
 
@@ -608,19 +766,37 @@ class MMProgressBar(MMWidget, gtk.Alignment):
         self._press_event = None
         self.update_state()
 
-        self.connect('track-seeked', callback)
+        self.bar.add_events(gtk.gdk.BUTTON_PRESS_MASK |
+            gtk.gdk.BUTTON_RELEASE_MASK |
+            gtk.gdk.POINTER_MOTION_MASK)
+
+        self._track_seeked_id = self.connect('track-seeked',
+            callback)
+        self._button_press_event_id = self.bar.connect('button-press-event',
+            self.on_button_press)
+        self._button_release_event_id = self.bar.connect('button-release-event',
+            self.on_button_release)
+        self._motion_notify_event_id = self.bar.connect('motion-notify-event',
+            self.on_motion_notify)
 
         event.add_callback(self.on_playback_state_change, 'playback_player_start')
         event.add_callback(self.on_playback_state_change, 'playback_toggle_pause')
         event.add_callback(self.on_playback_state_change, 'playback_player_end')
 
-        self.bar.add_events(gtk.gdk.BUTTON_PRESS_MASK |
-            gtk.gdk.BUTTON_RELEASE_MASK |
-            gtk.gdk.POINTER_MOTION_MASK)
+    def destroy(self):
+        """
+            Various cleanups
+        """
+        event.remove_callback(self.on_playback_state_change, 'playback_player_start')
+        event.remove_callback(self.on_playback_state_change, 'playback_toggle_pause')
+        event.remove_callback(self.on_playback_state_change, 'playback_player_end')
 
-        self.bar.connect('button-press-event', self.on_button_press)
-        self.bar.connect('button-release-event', self.on_button_release)
-        self.bar.connect('motion-notify-event', self.on_motion_notify)
+        self.disconnect(self._track_seeked_id)
+        self.bar.disconnect(self._button_press_event_id)
+        self.bar.disconnect(self._button_release_event_id)
+        self.bar.disconnect(self._motion_notify_event_id)
+
+        gtk.Alignment.destroy(self)
 
     def update_state(self):
         """
@@ -632,7 +808,7 @@ class MMProgressBar(MMWidget, gtk.Alignment):
         track = self.player.current
 
         if track is not None:
-            total = track.get_duration()
+            total = track.get_tag_raw('__length')
             current = self.player.get_progress() * total
             text = '%d:%02d / %d:%02d' % \
                 (current // 60, current % 60,
@@ -642,10 +818,10 @@ class MMProgressBar(MMWidget, gtk.Alignment):
                 self.disable_timer()
                 fraction = self.bar.get_fraction()
             elif self.player.is_playing():
-                if track.is_local() and track['__length']:
+                if track.get_tag_raw('__length'):
                     self.enable_timer()
                     fraction = self.player.get_progress()
-                else:
+                elif not track.is_local():
                     self.disable_timer()
                     text = _('Streaming...')
 
@@ -660,7 +836,7 @@ class MMProgressBar(MMWidget, gtk.Alignment):
         if self._timer is not None:
             return
 
-        self._timer = gobject.timeout_add(1000, self.update_state)
+        self._timer = gobject.timeout_add_seconds(1, self.update_state)
 
     def disable_timer(self):
         """
@@ -686,7 +862,7 @@ class MMProgressBar(MMWidget, gtk.Alignment):
             return True
 
         if not self.player.current.is_local() and \
-           not self.player.current['__length']:
+           not self.player.current.get_tag_raw('__length'):
             return True
 
         if event.button == 1:
@@ -735,30 +911,42 @@ class MMProgressBar(MMWidget, gtk.Alignment):
         self.bar.set_fraction(event.x / width)
         self.bar.set_text(text)
 
-gobject.type_register(MMProgressBar)
-gobject.signal_new('track-seeked', MMProgressBar,
+gobject.type_register(ProgressBar)
+gobject.signal_new('track-seeked', ProgressBar,
     gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
     (gobject.TYPE_FLOAT, ))
 
-class MMTrackBar(MMTrackSelector, MMProgressBar):
+class PlaylistProgressBar(gtk.HBox):
+    """
+        A progress bar with a button for
+        management of the current playlist
+    """
+    def __init__(self, main, queue, player, playlist, formatter,
+            change_callback, seeked_callback):
+        # button, bar
+        #button = PlaylistButton(main, queue, playlist, formatter, change_callback)
+        #self.pack_start(button, expand=False)
+        pass
+
+class TrackBar(TrackSelector, ProgressBar):
     """
         Track selector + progress bar = WIN
     """
     pass
 
-class MMTrackFormatter(gobject.GObject):
+class TrackFormatter(gobject.GObject):
     """
         Formats track titles based on a format string
     """
     __gsignals__ = {
-        'format-request': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_STRING, ()),
-        'rating-steps-request': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_INT, ()),
+        'format-changed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,))
     }
 
-    def __init__(self):
+    def __init__(self, format):
         gobject.GObject.__init__(self)
 
-        self._format = '$tracknumber - $title'
+        self._format = format
+        self._template = Template(self._format)
         self._substitutions = {
             'tracknumber': 'tracknumber',
             'title': 'title',
@@ -786,6 +974,30 @@ class MMTrackFormatter(gobject.GObject):
         }
         self._rating_steps = 5.0
 
+    def set_format(self, format):
+        """
+            Updates the internal template
+        """
+        if format == self._format:
+            return
+
+        self._format = format
+        self._template = Template(self._format)
+        self.emit('format-changed', self._format)
+
+    def set_rating_steps(self, rating_steps):
+        """
+            Updates the rating steps
+        """
+        if rating_steps == self._rating_steps:
+            return
+
+        self._rating_steps = rating_steps
+        rating_pattern = '%s%s' % (self._template.delimiter, 'rating')
+
+        if rating_pattern in self._format:
+            self.emit('format-changed', self._format)
+
     def format(self, track):
         """
             Returns the formatted title of a track
@@ -793,33 +1005,22 @@ class MMTrackFormatter(gobject.GObject):
         if not isinstance(track, Track):
             return None
 
-        template = Template(self.emit('format-request') or self._format)
-        text = template.safe_substitute(self.__get_substitutions(track))
-
-        return text
-
-    def __get_substitutions(self, track):
-        """
-            Returns a map for keyword to tag value mapping
-        """
         substitutions = self._substitutions.copy()
 
         for keyword, tagname in substitutions.iteritems():
-            try:
-                substitutions[keyword] = _(' & ').join(track[tagname])
-            except TypeError:
-                substitutions[keyword] = track[tagname]
+            substitutions[keyword] = track.get_tag_display(tagname,
+                artist_compilations=False)
 
             try:
-                formatter = self._formattings[keyword]
-                substitutions[keyword] = formatter(substitutions[keyword])
+                format_callback = self._formattings[keyword]
+                substitutions[keyword] = format_callback(substitutions[keyword])
             except KeyError:
                 pass
 
             if substitutions[keyword] is None:
                 substitutions[keyword] = _('Unknown')
 
-        return substitutions
+        return self._template.safe_substitute(substitutions)
 
     def __format_tracknumber(self, tracknumber):
         """
@@ -840,7 +1041,7 @@ class MMTrackFormatter(gobject.GObject):
         """
         try:
             length = float(length)
-        except TypeError:
+        except TypeError, ValueError:
             length = 0
 
         return '%d:%02d' % (length // 60, length % 60)
@@ -854,12 +1055,11 @@ class MMTrackFormatter(gobject.GObject):
         except TypeError:
             rating = 0
 
-        rating_steps = self.emit('rating-steps-request') or self._rating_steps
-        rating = rating_steps * rating
+        rating *= self._rating_steps
 
         return '%s%s' % (
             '★' * int(rating),
-            '☆' * int(rating_steps - rating)
+            '☆' * int(self._rating_steps - rating)
         )
 
     def __format_bitrate(self, bitrate):
@@ -870,6 +1070,8 @@ class MMTrackFormatter(gobject.GObject):
             bitrate = int(bitrate)
         except TypeError:
             bitrate = 0
+        except ValueError:
+            return bitrate
 
         return '%d kbit/s' % (bitrate / 1000)
 
@@ -879,6 +1081,11 @@ class MMTrackFormatter(gobject.GObject):
         """
         text = _('Never')
 
+        try:
+            last_played = float(last_played)
+        except ValueError:
+            last_played = None
+
         if last_played is not None:
             import time
             ct = time.time()
@@ -895,7 +1102,7 @@ class MMTrackFormatter(gobject.GObject):
                 text = _('Yesterday')
             else:
                 text = _('%(year)d-%(month)02d-%(day)02d') % {
-                    'year' : lptime.tm_year ,
+                    'year' : lptime.tm_year,
                     'month' : lptime.tm_mon,
                     'day' : lptime.tm_mday
                 }
