@@ -21,7 +21,7 @@ from xl.nls import gettext as _
 
 name = _('Streamripper')
 basedir = os.path.dirname(os.path.realpath(__file__))
-glade = '%s/streamripper.glade' % basedir
+ui = os.path.join(basedir, 'streamripper.ui')
 
 class SavePreference(widgets.DirPrefsItem):
     default = os.getenv('HOME')
@@ -30,3 +30,11 @@ class SavePreference(widgets.DirPrefsItem):
 class PortPreference(widgets.PrefsItem):
     default = '8888'
     name = 'plugin/streamripper/relay_port'
+    
+class FilePreference(widgets.CheckPrefsItem):
+    default = False
+    name = 'plugin/streamripper/single_file'
+    
+class DeletePreference(widgets.CheckPrefsItem):
+    default = True
+    name = 'plugin/streamripper/delete_incomplete'
